package org.eso.ohs.phase2.visiplot;

/**
 * Reporitory for Visibility Plot controls used for the display and calculation
 * of visibility plots.
 *
 * The VisiPlotOptions class stores all data as name and value pairs, and
 * can save/read data in from config files of PAF format.
 *
 * @author N. Kornweibel for ESO
 * <br>ChangeLog: cmavros 26/09/01 Add Time Interval menu option 
 */

import java.awt.Color;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.phase2.visibility.VisicalcOptions;


public class VisiPlotOptions {

    /**
     * Standard log channel
     * <br>ChangeLog: cmavros 21-12-01 added
     */	
    private static Logger stdlog_
            = Logger.getLogger( VisiPlotOptions.class );

    
    private static final long TwelveHoursInMilliseconds = 12 * 60 * 60 * 1000; 
    
    
    /*
	 * Visibility calculation options
	 */
	private VisicalcOptions visicalcOptions;
    
	
    /*
     * Sorting options
     */    
    private boolean groupOBs;
    private String sortPlot;
    
    /*
     * Report settings
     */
    
    private boolean reportShowSeeing;
    private boolean reportShowFLI;
    private boolean reportShowMoonDist;
    private boolean reportShowSkyTrans;
    private boolean reportShowAirMass;
    private boolean reportShowRank;
    private boolean reportShowProgId;
    private boolean reportShowOBId;
    private boolean reportShowRA;
    private boolean reportShowDec;
    private boolean reportShowExecTime;
    private boolean reportShowStartTime;
    private String reportPrinter;
    
    
    private String airmassValue;
    private String moonDistanceValue;
    private String fliValue;
    private String sunConstraint;
    
    
    /*
     * Color settings
     */    
    private Color airMassAreaColor;
    private Color airMassLineColor;
    private Color airMassLabelColor;
    
    private Color fliAreaColor;
    private Color fliLineColor;
    private Color fliLabelColor;
    
    private Color moonDistAreaColor;
    private Color moonDistLineColor;
    private Color moonDistLabelColor;
    
    private Color obexecAreaColor;    
    private Color obexecLineColor;
    private Color obexecLabelColor;
    
    private Color obTimeIntervalAreaColor;    
    private Color obTimeIntervalLineColor;
    private Color obTimeIntervalLabelColor;
    
    private Color obSiderealTimeIntervalAreaColor;
    private Color obSiderealTimeIntervalLineColor;
    private Color obSiderealTimeIntervalLabelColor;
    
    private Color sunAreaColor;
    private Color sunLineColor;
    private Color sunLabelColor;


    private boolean reportShowWind;      

        
    /**
     * Create a VisiPlotOptions object, with no options stored.
     */

    public VisiPlotOptions() {
    	visicalcOptions = new VisicalcOptions();
        setDefaults();
    }

    /**
     * Create a VisiPlotOptions object reading the input in PAF format
     * from the Reader object. Note the defaults settings are set
     * before over riding them from information from the PAF file.
     * @param the reader object containing the paf data for this.
     */

//    public VisiPlotOptions(Reader in) throws IOException {
//        setDefaults();
//        VisiplotOptionsHandler.readFrom(in,this);
//    }

    
	
  /**
   * @return true if this string indicates the value useOBValue_
   */

  public static boolean isReadFromOB(String str) {
      return VisiplotOptionsHandler.useOBValue.equalsIgnoreCase(str);
  }
  
  /**
   * @return true if val is an int between min and max
   * @param val the string containing the integer
   * @param min, minimum allowed value for val, inclusive.
   * @param max, maximum allowed value for val, inclusive.
   */
  
  public static boolean validInteger(String val, int min, int max) {
	boolean rc;
	try {
		int num = Integer.parseInt(val);
		rc = (num >= min && num <= max);
	} catch (NumberFormatException e) {
		rc = false;
	}
	return rc;
  }
  
  /**
   * @return true if val is a double between min and max
   * @param val the string containing the double
   * @param min, minimum allowed value for val, inclusive.
   * @param max, maximum allowed value for val, inclusive.
   */
  
  public static boolean validDouble(String val, double min, double max) {
	boolean rc;
	try {
		double num = Double.valueOf(val).doubleValue();
		rc = (num >= min && num <= max);
	} catch (NumberFormatException e) {
		rc = false;
	}
	return rc;
  }

  /**
   * Set the default values into the options class, clearing the store
   * before setting the new options.
   * Start and end times are set to today, 22h00 (UCT) start+12h respectively.
   */

  public void setDefaults() {

        Calendar today = Calendar.getInstance();
        // set to today.
        Date now = new Date();
        
        // subtract 12 hours so that "todays" date changes at 12h UTC, instead
        // of 0h UTC. This is a request (bug 2565) from Paranal and means that
        // "today" does not change halfway through the observing night.
        // Note the day will now change at 12 midday GMT - for Paranal this
        // means around 8am in the morning....which is acceptable.
        now.setTime(now.getTime() - TwelveHoursInMilliseconds);
        // debugging for the above problem.
        stdlog_.debug("Default plot date is the day of: "+now.toString());
        
        today.setTime(now);
        // set to 10pm UTC this evening
        today.set(Calendar.HOUR_OF_DAY, 22);
        today.set(Calendar.MINUTE, 0);
        today.set(Calendar.SECOND, 0);
        Date startTime = today.getTime();
        // set end time 12h later than the 22h start time.
        Date endTime = new Date(startTime.getTime()+(TwelveHoursInMilliseconds));
  
        visicalcOptions = new VisicalcOptions();
        
        visicalcOptions.setStartTime(startTime);
        visicalcOptions.setEndTime(endTime);
        setGroupOBs(true);
        setSortPlot(VisiplotOptionsHandler.defaultSortOrder); 
        
        setAirmassValue(VisiplotOptionsHandler.useOBValue );
        setMoonDistanceValue(VisiplotOptionsHandler.useOBValue );
        setFliValue(VisiplotOptionsHandler.useOBValue );
        
        visicalcOptions.setCalcAirmass(true);    
        visicalcOptions.setCalcMoon(true);
        visicalcOptions.setCalcMoonDist(true);                        
        visicalcOptions.setCalcFli(true);        
        visicalcOptions.setCalcExecTime(true);        
        visicalcOptions.setCalcTimeInterval(false);
        visicalcOptions.setCalcSiderealTimeInterval(false);        
        visicalcOptions.setCalcSun(true);  
        visicalcOptions.setSunConstraint(VisiplotOptionsHandler.defaultSunCon);                        
        visicalcOptions.setInterval(VisiplotOptionsHandler.defaultPrecision );        
        
        
        setReportShowWind(true);
        setReportShowSeeing(true);
        setReportShowFLI(false);
        setReportShowMoonDist(false);
        setReportShowSkyTrans(true);
        setReportShowAirMass(true);
        setReportShowRank(true);
        setReportShowProgId(true);
        setReportShowOBId(true);
        setReportShowRA(true);
        setReportShowDec(true);
        setReportShowExecTime(true);
        setReportShowStartTime(true);
        setReportPrinter("");
        
        // set the colors
        ChartColorItem airMass = new ChartColorItem(VisiplotOptionsHandler.AIR_MASS_CHART_LABEL, true, true);
        ChartColorItem moonDist = new ChartColorItem(VisiplotOptionsHandler.MOON_DISTANCE_CHART_LABEL, true, true);
        ChartColorItem fli = new ChartColorItem(VisiplotOptionsHandler.FRACTIONAL_LUNAR_ILL_CHART_LABEL, true, true);
        ChartColorItem sun = new ChartColorItem(VisiplotOptionsHandler.SUN_ANGLE_CHART_LABEL, true, true);
        ChartColorItem timeInterval = new ChartColorItem(VisiplotOptionsHandler.TIME_INTERVALS_CHART_LABEL, true, true);
        ChartColorItem execTime = new ChartColorItem(VisiplotOptionsHandler.EXECUTION_TIME_CHART_LABEL, true, true);
        ChartColorItem sidereal = new ChartColorItem(VisiplotOptionsHandler.ST_TIME_INTERVALS_CHART_LABEL, true, true);
        
        airMass = ChartColorItem.setChartColorItem(airMass,true,true,null,null,null);
        moonDist = ChartColorItem.setChartColorItem(moonDist,true,true,null,null,null);
        fli = ChartColorItem.setChartColorItem(fli,true,true,null,null,null);
        sun = ChartColorItem.setChartColorItem(sun,true,true,null,null,null);
        timeInterval = ChartColorItem.setChartColorItem(timeInterval,true,true,null,null,null);
        execTime = ChartColorItem.setChartColorItem(execTime,true,true,null,null,null);
        sidereal = ChartColorItem.setChartColorItem(sidereal,true,true,null,null,null);        
        
        setAirMassAreaColor(airMass.getAreaColor());
        setAirMassLineColor(airMass.getPerimeterColor());
        setAirMassLabelColor(airMass.getLabelsColor());
        
        
        setMoonDistAreaColor(moonDist.getAreaColor());
        setMoonDistLineColor(moonDist.getPerimeterColor());
        setMoonDistLabelColor(moonDist.getLabelsColor());
        
        setFliAreaColor(fli.getAreaColor());
        setFliLineColor(fli.getPerimeterColor());
        setFliLabelColor(fli.getLabelsColor());
        
        setSunAreaColor(sun.getAreaColor());
        setSunLineColor(sun.getPerimeterColor());
        setSunLabelColor(sun.getLabelsColor());
        
        setObTimeIntervalAreaColor(timeInterval.getAreaColor());
        setObTimeIntervalLineColor(timeInterval.getPerimeterColor());
        setObTimeIntervalLabelColor(timeInterval.getLabelsColor());
        
        setObexecAreaColor(execTime.getAreaColor());
        setObexecLineColor(execTime.getPerimeterColor());
        setObexecLabelColor(execTime.getLabelsColor());
        
        setObSiderealTimeIntervalAreaColor(sidereal.getAreaColor());
        setObSiderealTimeIntervalLineColor(sidereal.getPerimeterColor());
        setObSiderealTimeIntervalLabelColor(sidereal.getLabelsColor());

  }
  
  
  public Date getEndTime() {
	  return(visicalcOptions.getEndTime());
  }
  
  public String getEndTimeISO() {
	  Date date = visicalcOptions.getEndTime();
	  return formatISO(date);
  }

  private String formatISO(Date date) {
		SimpleDateFormat formatter = new SimpleDateFormat(
				Convert.ISO_DATE_FORMAT);
		return formatter.format(date);
	}
  
  
  public Date getStartTime() {
	  return(visicalcOptions.getStartTime());
  }
  
  public String getStartTimeISO() {
	  Date date = visicalcOptions.getStartTime();
	  return formatISO(date);
  }
  
  public int getInterval() {
	  return(visicalcOptions.getInterval());
  }

	public boolean isGroupOBs() {
		return groupOBs;
	}

	public void setGroupOBs(boolean groupOBs) {
		this.groupOBs = groupOBs;
	}

	public VisicalcOptions getVisicalcOptions() {
		return visicalcOptions;
	}

	public String getSortPlot() {
		return sortPlot;
	}

	public void setSortPlot(String sortPlot) {
		this.sortPlot = sortPlot;
	}

	public boolean reportShowExecTime() {
		return reportShowExecTime;
	}

	public void setReportShowExecTime(boolean reportExecTime) {
		this.reportShowExecTime = reportExecTime;
	}

	public String getReportPrinter() {
		return reportPrinter;
	}

	public void setReportPrinter(String reportPrinter) {
		this.reportPrinter = reportPrinter;
	}

	public boolean reportShowAirMass() {
		return reportShowAirMass;
	}

	public void setReportShowAirMass(boolean reportShowAirMass) {
		this.reportShowAirMass = reportShowAirMass;
	}

	public boolean reportShowDec() {
		return reportShowDec;
	}

	public void setReportShowDec(boolean reportShowDec) {
		this.reportShowDec = reportShowDec;
	}

	public boolean reportShowFLI() {
		return reportShowFLI;
	}

	public void setReportShowFLI(boolean reportShowFLI) {
		this.reportShowFLI = reportShowFLI;
	}

	public boolean reportShowMoonDist() {
		return reportShowMoonDist;
	}

	public void setReportShowMoonDist(boolean reportShowMoonDist) {
		this.reportShowMoonDist = reportShowMoonDist;
	}

	public boolean reportShowOBId() {
		return reportShowOBId;
	}

	public void setReportShowOBId(boolean reportShowOBId) {
		this.reportShowOBId = reportShowOBId;
	}

	public boolean reportShowProgId() {
		return reportShowProgId;
	}

	public void setReportShowProgId(boolean reportShowProgId) {
		this.reportShowProgId = reportShowProgId;
	}

	public boolean reportShowRA() {
		return reportShowRA;
	}

	public void setReportShowRA(boolean reportShowRA) {
		this.reportShowRA = reportShowRA;
	}

	public boolean reportShowRank() {
		return reportShowRank;
	}

	public void setReportShowRank(boolean reportShowRank) {
		this.reportShowRank = reportShowRank;
	}

	public boolean reportShowSeeing() {
		return reportShowSeeing;
	}

    public void setReportShowSeeing(boolean reportShowSeeing) {
        this.reportShowSeeing = reportShowSeeing;
    }
    public boolean reportShowWind() {
        return reportShowWind;
    }

    public void setReportShowWind(boolean reportShowWind) {
        this.reportShowWind = reportShowWind;
    }

	public boolean reportShowSkyTrans() {
		return reportShowSkyTrans;
	}

	public void setReportShowSkyTrans(boolean reportShowSkyTrans) {
		this.reportShowSkyTrans = reportShowSkyTrans;
	}

	public boolean reportShowStartTime() {
		return reportShowStartTime;
	}

	public void setReportShowStartTime(boolean reportStartTime) {
		this.reportShowStartTime = reportStartTime;
	}

	public String getAirmassValue() {
		return airmassValue;
	}

	public void setAirmassValue(String airmassValue) {
		this.airmassValue = airmassValue;
	}

	public String getFliValue() {
		return fliValue;
	}

	public void setFliValue(String fliValue) {
		this.fliValue = fliValue;
	}

	public String getMoonDistanceValue() {
		return moonDistanceValue;
	}

	public void setMoonDistanceValue(String moonDistanceValue) {
		this.moonDistanceValue = moonDistanceValue;
	}

	

	public String getSunConstraint() {
		return sunConstraint;
	}

	public void setSunConstraint(String sunConstraint) {
		this.sunConstraint = sunConstraint;
	}

	public Color getAirMassAreaColor() {
		return airMassAreaColor;
	}

	public void setAirMassAreaColor(Color airMassAreaColor) {
		this.airMassAreaColor = airMassAreaColor;
	}

	public Color getAirMassLabelColor() {
		return airMassLabelColor;
	}

	public void setAirMassLabelColor(Color airMassLabelColor) {
		this.airMassLabelColor = airMassLabelColor;
	}

	public Color getAirMassLineColor() {
		return airMassLineColor;
	}

	public void setAirMassLineColor(Color airMassLineColor) {
		this.airMassLineColor = airMassLineColor;
	}

	public Color getFliAreaColor() {
		return fliAreaColor;
	}

	public void setFliAreaColor(Color fliAreaColor) {
		this.fliAreaColor = fliAreaColor;
	}

	public Color getFliLabelColor() {
		return fliLabelColor;
	}

	public void setFliLabelColor(Color fliLabelColor) {
		this.fliLabelColor = fliLabelColor;
	}

	public Color getFliLineColor() {
		return fliLineColor;
	}

	public void setFliLineColor(Color fliLineColor) {
		this.fliLineColor = fliLineColor;
	}

	
	public Color getObexecAreaColor() {
		return obexecAreaColor;
	}

	public void setObexecAreaColor(Color obexecAreaColor) {
		this.obexecAreaColor = obexecAreaColor;
	}

	public Color getObexecLabelColor() {
		return obexecLabelColor;
	}

	public void setObexecLabelColor(Color obexecLabelColor) {
		this.obexecLabelColor = obexecLabelColor;
	}

	public Color getObexecLineColor() {
		return obexecLineColor;
	}

	public void setObexecLineColor(Color obexecLineColor) {
		this.obexecLineColor = obexecLineColor;
	}

	public Color getObSiderealTimeIntervalAreaColor() {
		return obSiderealTimeIntervalAreaColor;
	}

	public void setObSiderealTimeIntervalAreaColor(
			Color obSiderealTimeIntervalAreaColor) {
		this.obSiderealTimeIntervalAreaColor = obSiderealTimeIntervalAreaColor;
	}

	public Color getObSiderealTimeIntervalLabelColor() {
		return obSiderealTimeIntervalLabelColor;
	}

	public void setObSiderealTimeIntervalLabelColor(
			Color obSiderealTimeIntervalLabelColor) {
		this.obSiderealTimeIntervalLabelColor = obSiderealTimeIntervalLabelColor;
	}

	public Color getObSiderealTimeIntervalLineColor() {
		return obSiderealTimeIntervalLineColor;
	}

	public void setObSiderealTimeIntervalLineColor(
			Color obSiderealTimeIntervalLineColor) {
		this.obSiderealTimeIntervalLineColor = obSiderealTimeIntervalLineColor;
	}

	public Color getObTimeIntervalAreaColor() {
		return obTimeIntervalAreaColor;
	}

	public void setObTimeIntervalAreaColor(Color obTimeIntervalAreaColor) {
		this.obTimeIntervalAreaColor = obTimeIntervalAreaColor;
	}

	public Color getObTimeIntervalLabelColor() {
		return obTimeIntervalLabelColor;
	}

	public void setObTimeIntervalLabelColor(Color obTimeIntervalLabelColor) {
		this.obTimeIntervalLabelColor = obTimeIntervalLabelColor;
	}

	public Color getObTimeIntervalLineColor() {
		return obTimeIntervalLineColor;
	}

	public void setObTimeIntervalLineColor(Color obTimeIntervalLineColor) {
		this.obTimeIntervalLineColor = obTimeIntervalLineColor;
	}

	public Color getSunAreaColor() {
		return sunAreaColor;
	}

	public void setSunAreaColor(Color sunAreaColor) {
		this.sunAreaColor = sunAreaColor;
	}

	public Color getSunLabelColor() {
		return sunLabelColor;
	}

	public void setSunLabelColor(Color sunLabelColor) {
		this.sunLabelColor = sunLabelColor;
	}

	public Color getSunLineColor() {
		return sunLineColor;
	}

	public void setSunLineColor(Color sunLineColor) {
		this.sunLineColor = sunLineColor;
	}

	public Color getMoonDistAreaColor() {
		return moonDistAreaColor;
	}

	public void setMoonDistAreaColor(Color moonDistAreaColor) {
		this.moonDistAreaColor = moonDistAreaColor;
	}

	public Color getMoonDistLabelColor() {
		return moonDistLabelColor;
	}

	public void setMoonDistLabelColor(Color moonDistLabelColor) {
		this.moonDistLabelColor = moonDistLabelColor;
	}

	public Color getMoonDistLineColor() {
		return moonDistLineColor;
	}

	public void setMoonDistLineColor(Color moonDistLineColor) {
		this.moonDistLineColor = moonDistLineColor;
	}
	
	
	



}
