//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.2 $
//$Log: DfcExceptions.h,v $
//Revision 1.2  2004/10/26 15:23:54  nferguso
//Temporary check-in for repository move.
//
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:20  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:15  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:06  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:42  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:50  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:12  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_EXCEPTIONS_H
#define DFC_EXCEPTIONS_H

#include <iostream>
#include "DfcGlobals.h"


//
// Global that is used akin to errno. Each Exception has a code. When an
// exception is thrown this global status variable records which exception 
// has been thrown. This is because g++ doesn't appear to work properly with
// regards to exceptions. For example if we throw an exception that is derived
// from a base exception class and then attempt to catch a reference to the
// base class it will not catch the derived class. 
// If an exception remains uncaught then g++ generates an 'Abort' error and
// dumps core. Therefore every try block must be followed by a catch(...)
// statement (akin to a case default in a switch) which should indicate which
// exception has been thrown but not caught. For this I need a global status
// cariable.
// It's defined in DfcExceptions.C to avoid linkage problems
//
extern DFC_STRING DfcExceptionLastExc;

//
// Exception classes used for this file
//
///
class DfcException {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcExceptions.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

  {\large DESCRIPTION} \\
  Top level exception class for the Frame subsystem . This is the parent class for ALL exceptions that are generated by Tables, Frames, Images, Keywords etc.
  At some point it should itself be derived from a system wide exception class which provides base functionality or a standard interface. \\
  Note that these excpetion classes have not been given any data or methods at the moment. This is because it is possible to do this or it is possible merely to store a string in the Exception base class and provide a report() method. I'm preserving judgement at the moment and doing some literature searches on the subject. \\
  Ideally we want to limit the number of exceptions that can be thrown so that every case can be tested for in its own catch block. We want the exceptions to be general enough so that we don't have to test for hundreds of exceptions, and yet precise enough to be of real use in the cases where remedial action can be taken.
  */
};

///
class DfcFrameExc: public DfcException {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcExceptions.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

  {\large DESCRIPTION} \\
  Top level exception class for the Frame class. This may be useful in conjuntion with RTTI at some point, it may not. I haven't come across much written thought on Exception design yet - so this is only a try to see how things go.
  */
public:
  friend std::ostream &operator<<(std::ostream &out, const DfcFrameExc &data);

  DfcFrameExc(void);
  DfcFrameExc(const DfcFrameExc &other);
  DfcFrameExc(const DFC_STRING &where);
  DfcFrameExc operator=(const DfcFrameExc &other);
  DFC_STRING getThrower(void) const;
  virtual DFC_STRING report(void) const;
  virtual ~DfcFrameExc(void);

protected:

private:
  DFC_STRING where;
};

///
class DfcBoundsExc: public DfcFrameExc {
public:
  DfcBoundsExc(void);
  DfcBoundsExc(const DfcBoundsExc &other);
  DfcBoundsExc(const DFC_STRING &where, int requested, int max);
  DfcBoundsExc &operator=(const DfcBoundsExc &other);
  virtual DFC_STRING report(void) const;
  virtual ~DfcBoundsExc(void);
protected:

private:
  int max;
  int requested;
};

///
class DfcDoesntExistExc: public DfcFrameExc {
public:
  DfcDoesntExistExc(void);
  DfcDoesntExistExc(const DfcDoesntExistExc &other);
  DfcDoesntExistExc(const DFC_STRING &where, const DFC_STRING &what);
  DfcDoesntExistExc &operator=(const DfcDoesntExistExc &other);
  virtual DFC_STRING report(void) const;
  virtual ~DfcDoesntExistExc(void);
protected:

private:
  DFC_STRING whatExists;
};

///
class DfcAlreadyOpenExc: public DfcFrameExc {};

///
class DfcComponentOpenExc: public DfcAlreadyOpenExc {};
/**
May be useful later. For example, if we open a Frame that contains Table References, but one of the Tables has already been opened then it might be sensible to generate this exception instead of the parent one.
 */

///
class DfcReadOnlyExc: public DfcFrameExc {};

///
class DfcAlreadyExistsExc: public DfcFrameExc {
public:
  DfcAlreadyExistsExc(void);
  DfcAlreadyExistsExc(const DfcAlreadyExistsExc &other);
  DfcAlreadyExistsExc(const DFC_STRING &where, const DFC_STRING &what);
  DfcAlreadyExistsExc &operator=(const DfcAlreadyExistsExc &other);
  virtual DFC_STRING report(void) const;
  virtual ~DfcAlreadyExistsExc(void);
protected:

private:
  DFC_STRING whatExists;
};

///
class DfcUnsetExc: public DfcFrameExc {
public:
  DfcUnsetExc(void);
  DfcUnsetExc(const DfcUnsetExc &other);
  DfcUnsetExc(const DFC_STRING &where, int whichOne);
  DfcUnsetExc &operator=(const DfcUnsetExc &other);
  virtual DFC_STRING report(void) const;
  virtual ~DfcUnsetExc(void);
protected:

private:
  int whichElement;
};

///
class DfcMemoryExc: public DfcFrameExc {
public:
  DfcMemoryExc(void);
  DfcMemoryExc(const DfcMemoryExc &other);
  DfcMemoryExc(const DFC_STRING &where);
  DfcMemoryExc &operator=(const DfcMemoryExc &other);
  virtual ~DfcMemoryExc(void);
protected:

private:

};

///
class DfcTypeExc: public DfcFrameExc {
public:
  DfcTypeExc(void);

  DfcTypeExc(
  ///
  const DFC_STRING &throwerClass,
  ///
  const DFC_STRING &throwerName,
  ///
  const DFC_STRING &throwerType,
  ///
  const DFC_STRING &accessType
  );

  ///
  DFC_STRING report(void) const;

  ///
  virtual ~DfcTypeExc(void);

protected:

private:
  DFC_STRING throwerName;
  DFC_STRING throwerType;
  DFC_STRING accessType;
};

///
class DfcBadPixelExc: public DfcFrameExc {};

/// 
class DfcDimensionsExc: public DfcFrameExc {};

///
class DfcInvalidExprExc: public DfcFrameExc {};

/// to be removed after implementation is complete
class DfcNotImplementedYetExc: public DfcFrameExc {
public:
  DfcNotImplementedYetExc(void);
  DfcNotImplementedYetExc(const DFC_STRING &where);
  DfcNotImplementedYetExc(const DfcNotImplementedYetExc &other);
  DfcNotImplementedYetExc operator=(const DfcNotImplementedYetExc &other);
  ~DfcNotImplementedYetExc(void);
protected:
private:
};

///
class DfcInvalidStreamExc: public DfcFrameExc {};

///
void DfcAbort(const DfcFrameExc &exception);


#endif // DFC_EXCEPTIONS_H

//
//eof
//







