//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.1 $
//$Log: DfcKeyword.h,v $
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:20  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:15  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:03  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:46  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:51  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_KEYWORD_H
#define DFC_KEYWORD_H

// includes from this library
#include "DfcKeywordInterface.h"
#include "DfcKeywordReal32.h"
#include "DfcKeywordReal64.h"
#include "DfcKeywordInt2C64.h"
#include "DfcStringKeyword.h"
#include "DfcKeywordStructure.h"


//
// some hackery to get the thing to compile rather than ripping everything out
// of the interface.
//
class DfcColumnInterface {};
class DfcRow {};

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class


///
class DfcKeyword: public DfcKeywordInterface{
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcKeyword.h,v 1.1 2004/09/06 14:35:22 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
  \end{verbatim}

  {\large DESCRIPTION} \\
  Keywords contain arbitrary pieces of information and can be found attached to various objects such as Frames, Columns, Tables, Images....\\
  Keywords can hold the following: \\
  1 - a name \\
  2 - a descriptive piece of text about the Keyword \\
  3 - either: \\
  a number of data fiels, all of the same type \\
  OR\\
  a number of nested Keywords. \\
  \\
  Therefore we can have a nested tree structure of Keywords.

  {\large EXAMPLES} \\
  \begin{verbatim}
  // 
  // We have an image object called myImage
  // We want to obtain the data as an int 
  // 
  try
  {
    DFC_INT_UNSIGN_16 val = myImage.getKeywords()("firstStruct")("secondStruct").getAsIntUnsign_16();
  }
  catch(DfcDoesntExistExc &exception)
  {
    exception.report();
  }

  // now add a new Keyword to the heirarchy
  int fiveElements = 5; 
  try
  {
    DfcIntUnsign16_Keyword newKeyword("uniqueName", "no description", fiveElements); 
  }
  catch (DfcMemoryExc)
  {
    cerr << "out of memory, keyword not created\n";
  }
  try
  {
    myImage.getKeywords().addKeyword(newKeyword); 
  }
  catch(DfcTypeExc &exception)
  {
    cerr << "This Keyword doesn't contain other Keywords\n";
  }
  \end{verbatim}

  {\large DESIGN DECISIONS} \\
  There is the possibility here of not differentiating between KeywordStructures and KeywordData. A KeywordStructure is in this case simply a KeywordData object that holds a number of Keywords. This would remove the possibility of attempting to invoke a method and an exception being raised because we are dealing with the wrong type of Keyword. Personally I feel it is better to make the differentiation. For navigation it is simpler to have the differentiation, it yields nicer OO Style code. \\
  I have moved a lot of access functions into the base class so that data can be accessed as any type. There are two ways of doing this. Say we want to write a method that allows us to get the data as an int. we can write:\\
  int getAsInt(void) const;\\
  or \\
  void get(int &returnedValue); \\
  \\
  Personally I prefer the first as it makes one be explicit about types, as well as providing the return value in a syntactically more convenient position. However, the use of overloading in the second version makes for more type transparency which appears to be a major desire throughout this subsystem. I would have opted for this approach.....\\
  ....but....\\
  ... seeing as we have DFC_INT_32, DFC_INT_64 etc for scalar types, it won't work. It may be that we ship to an architecture that doesn't support one of these types and so we have to use the nearest available type. Most likely these will be typedef'd, so both of the above may be typedef'd to double. Then we have two methods overloaded with the same type which just won't work:\\
  void get(DFC_REAL_32 &returnedValue) const; \\
  and \\
  void get(DFC_REAL_64 &returnedValue) const; // error, redeclaration of void get(double &returnedValue) const;\\
*/

public:
  ///
  DfcKeyword(
  ///
  DfcKeywordInterface *realKeyword
  /**
  The actual keyword object that this proxy refers to.
   */
  );
  // throw();
  /**
  {\large Purpose}\\
  Create a proxy for the supplied object. 
   */

  ///
  DfcKeyword(
  ///
  const DFC_STRING &name,
  /**
  The name for the Column. 
   */
  ///
  const DfcVector<double> *data
  /**
  Obtain the data from a Vector.
   */
  );
  // throw(DfcMemoryExc);
  /** 
  {\large Purpose} \\
  Create a new Keyword, taking the data from a Vector. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there wasn't enough memory on the local machine to create the Keyword.
  \end{itemize}
  */

  ///
  DfcKeyword(
  ///
  const DFC_STRING &name,
  /**
  The name for the Column 
  */
  ///
  const DfcColumnInterface *data
  /**
  obtain the data for the column 
   */
  ///
  );
  // throw(DfcMemoryExc);
  /** 
  {\large Purpose} \\
  Create a new Keyword, taking the data from a Column. Only those Rows in the Column that are selected shall be used.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there wasn't enough memory on the local machine to create the Keyword.
  \end{itemize}
  */

  ///
  DfcKeyword(
  ///
  const DFC_STRING &name,
  /**
  The name for the Column 
  */
  ///
  const DfcRow *data
  /**
  obtain the data for the column 
   */
  );
  // throw(DfcMemoryExc);
  /** 
  {\large Purpose} \\
  Create a new KeywordStructure using the data from a Row. Only thos Columns that are selected from the Row shall be used to form the Keyword. The Keyword data contained within the Keyword structure shall have the same name as the Columns from which tyhey are taken.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there wasn't enough memory on the local machine to create the Keyword.
  \end{itemize}
  */

  ///
  DfcKeyword(void);
  // throw();
  /** {\large Purpose} \\
  Default Constructor.
  */

  ///
  DfcKeyword(
  ///
  const DfcKeyword &other
  );

  ///
  DfcKeyword(
  ///
  const DfcKeywordInterface &other
  /**
  Reference to the other Keyword
  */
  );
  // throw();
  /**
  {\large Purpose}\\
  Copy constructor.
  */

  ///
  DfcKeyword(
  ///
  const DFC_STRING &name, 
  ///
  DfcKeywordInterface::TYPE type, 
  ///
  const DFC_STRING &description, 
  ///
  const DFC_STRING &alias,
  ///
  int numEls = 1
  );

  ///
  DfcKeyword(
  ///
  const DFC_STRING &name, 
  ///
  DfcKeywordInterface::TYPE type, 
  ///
  int numEls = 1
  );

  ///
  int getNumElements(void) const;
  // throw();
  /**
  {\large Purpose} \\
  Returns the number of data elements in the Keyword. 

  {\large Returns}\\
  the number of data elements.
  */

  ///
  int getNumKeywords(void) const;
  // throw();
  /** {\large Purpose} \\
  Returns the number of Keywords that this Keyword contains. 

  {\large Returns}\\
  the number of keywords.

  */
  
  ///
  DFC_STRING getName(void) const;
  // throw();
  /**
  {\large Purpose}\\
  Returns the name of this Keyword.
   */

  ///
  DFC_STRING getDescription(void) const;
  // throw();
  /**
  {\large Purpose}\\
  Returns the description associated with this Keyword.
   */

  ///
  void setDescription(
  ///
  const DFC_STRING &newDesc
  /**
  A new descriptive text string for the Keyword.
   */
  );
  // throw();
  /**
  {\large Purpose}\\
  Set a descriptive text field in the Keyword. This forms part of the report produced by the report() method.
  */

  ///
  void setName(
  ///
  const DFC_STRING &newName
  /**
  The new name for the Keyword.
   */
  );
  // throw(DfcAlreadyExistsExc);
  /** {\large Purpose} \\
  Set the name of the Keyword. Each Keyword in a KeywordStructure must have a unique name, this is enforced.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - the new name for the Keyword already exists within the containing Keyword which is not allowed. The attempt to set the name has failed and the name remains as it was.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordInterface::TYPE getType(void) const;
  // throw(); 
  /** {/large Purpose} \\
  An indication of the type of the Keyword. This is used while the standard C++ RTTI mechanisms are still in their infancy and may, or may not, become redundant later. The primary purpose of the type field is to allow persistency, not so that users can query it and perform switch statements on the type. Because of this it may be hidden from the public interface later.
  */

  ///
  DFC_STRING report(void) const;
  // throw();
  /** {\large Purpose} \\
  Output some information about this Keyword, probably it's name, type, description and stuff like that.
 
  {\ large Returns} \\
  DFC_STRING instance containing the report.
   */

  ///
  DfcKeyword *operator()(
  ///
  const DFC_STRING &target
  /**
  A string for a Keyword to search for in the current Keyword.  
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct("instruments")("atmosphere")("seeing").report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void addCopyOfKeyword(
  /// 
  DfcKeywordInterface &newKeyword
  /**
  A reference to the Keyword to be added into the current Keyword. This Keyword will be copied into the KeywordStructure so the responsibility for the deletion of this argument still lies with the method invoker. Likewise, further changes of this argument will not result in the newly added Keyword being changed. 
  */
  );
  // throw(DfcAlreadyExistsExc, DfcTypeExc);
  /** {\large Purpose} \\
  Add a new Keyword (Structure or Data) to the current Keyword.
 
  {\large Preconditions}\\
  The current Keword should really be a KeywordStructure if this method is to succeed.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - The name already exists in the current Keyword. Names should be unique in any given KeywordStructure.
  \item DfcTypeExc - the Keyword that we're attempting to add the Keyuword into does not contain Keywords !?!
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void deleteKeyword(
  ///
  const DFC_STRING &target
  /**
  The keyword to find and remove from the current Keyword.
   */
  );
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find and remove the specified Keyword from the current Keyword. NB if you have obtained a pointer to this keyword note that it will no longer be valid eg \\
  \begin{verbatim}\\
  DfcKeyword *k1 = myStructure("keywordToBeFound");
  myStructure.delete("keywordToBeFound");
  cout << k1->getName() << endl;  // invalid pointer use
  \end{verbatim}
 
  {\large Preconditions}\\
  the current Keyword should be a KeywordStructure object.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExist - the requested Keyword for removal was not found in the current Keyword (it may be that this Keyword does not contain other Keywords).
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcKeywordInterface *getParent(void) const;
  // throw()
  /**
  {/large Purpose} \\
  Discover the parent Kewyword of this Keyword. This method is provided with the intent of finding out where in the hierarchy of Keywords one is, not so that the hierarchy can be navigated backwards.

  {/large Returns} \\
  const pointer to the parent object if this Keyword has a parent, otherwise to Null.
  */

  ///
  DFC_REAL_32 getAsReal32(
  ///
  int whichElement = 0
  /**
  Each Keyword can contain several items of data (of the same type). This field specifies which one is being requested.
   */
  ) const;
  // throw(DfcTypeExc, DfcBoundsExc);
  /** {\large Purpose} \\
  Obtain data from the Keyword as a 32 bit real value.

  {\large Preconditions}
  0 <= whichElement < this::getNumElements()

  {\large Returns}\\
  A 32 bit real.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - the data cannot be obtained as a 32 bit real. For example, this Keyword contains Time objects or other Keywords.
  \item DfcBoundsExc - the requested element is not a valid one.
  \end{itemize}

  {\large Arguments}\\
  */


  ///
  DFC_REAL_64 getAsReal64(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DFC_INT_UNSIGN_8 getAsIntUnsign8(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DFC_INT_UNSIGN_16 getAsIntUnsign16(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DFC_INT_SIGN_16 getAsIntSign16(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DFC_INT_2C_32 getAsInt2C_32(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DFC_INT_2C_64 getAsInt2C_64(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  char getAsChar(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DFC_STRING getAsString(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DfcCoordSys getAsCoordSys(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DfcTime getAsTime(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DfcFrame getAsFrameRef(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DfcTable getAsTableRef(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DfcEnumerated getAsEnumerated(int whichElement = 0) const;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  DfcTarget getAsTarget(int whichElement = 0) const;  
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsReal32(DFC_REAL_32 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsReal64(DFC_REAL_64 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsIntUnsign8(DFC_INT_UNSIGN_8 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsIntUnsign16(DFC_INT_UNSIGN_16 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsIntSign16(DFC_INT_SIGN_16 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsInt2C_32(DFC_INT_2C_32 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsInt2C_64(DFC_INT_2C_64 newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsChar(char newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsString(const DFC_STRING &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsCoordSys(const DfcCoordSysInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsTime(const DfcTimeInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsFrameRef(DfcFrameInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsTableRef(DfcTableInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsEnumerated(const DfcEnumeratedInterface &newValue, int whichElement = 0);
  // throw(DfcTypeExc, DfcBoundsExc);

  ///
  void setAsTarget(const DfcTargetInterface &newValue, int whichElement = 0);  
  // throw(DfcTypeExc, DfcBoundsExc);

  
  ///
  DfcKeywordInterface *clone(void) const;
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Make a copy of this object. Altering the copy will not affect this object in any way. (technical - we make a deep copy of this object.)  

  {\large Returns}\\
  A pointer to the copy of this object. The responsibility for the memory management of this copy now lies with the method invoker.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  ///
  DFC_BOOL operator==(const DfcKeywordInterface &other) const;
  /** {\large Purpose} \\
  Equality operator (associated operators must be defined later. Only performs comparisons on the basis of the name field. This is how the objects are to be uniquely identified.
  */

  /// 
  DfcKeywordInterface &operator=(const DfcKeyword &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  /// 
  DfcKeywordInterface &operator=(const DfcKeywordInterface &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  /// 
  DFC_BOOL write(
  ///
  const DfcIO_Strategy &strategy
  /**
  A DfcIO_Strategy instance that dictates how and where the object writes itself.
   */
  );
  // throw(DfcReadOnlyExc);
  /**
  {\large Purpose} \\
  Makes the object write itself. The object writes itself in terms of calls to the methods in the DfcIO_Strategy instance.
  
  {\large Returns}\\
  Boolean indication of whether the saving of the data was succesful or not.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcReadOnlyExc - the class cannot be written back because it was opened for read-only access.
  \end{itemize}

  {\large Arguments}\\
  */

  /// 
  DFC_BOOL openForRead(
  ///
  const DfcIO_Strategy &strategy
  /**
  A DfcIO_Strategy instance from which this object requests its data.
  */
  );  
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /**
  {\large Purpose} \\
  Makes the Keyword instance read itself from file / server / whatever. The Keyword object only needs to make invocations of the methods in the supplied IO_Strategy object to obtain its data. In this way the source and format of the data is completely transparent to it.
  
  {\large Returns}\\
  Boolean indicating whether the data was succesfully read or not.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - while trying to create the sub-objects to hold the data locally, not enough memory was available.
  \item DfcAlreadyOpenExc - the object is already open for updating.
  \end{itemize}

  {\large Arguments}\\
  */

  /// 
  DFC_BOOL openForUpdate(
  ///
  const DfcIO_Strategy &strategy
  /**
  A DfcIO_Strategy instance from which this object requests its data.
  */
  );  
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /**
  {\large Purpose} \\
  Makes the Keyword instance read itself from file / server / whatever. The Keyword object only needs to make invocations of the methods in the supplied IO_Strategy object to obtain its data. In this way the source and format of the data is completely transparent to it.
  
  {\large Returns}\\
  Boolean indicating whether the data was succesfully read or not.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - while trying to create the sub-objects to hold the data locally, not enough memory was available.
  \item DfcAlreadyOpenExc - the object is already open somewhere.
  \end{itemize}

  {\large Arguments}\\
  */

  // conform to DfcNumericalInterface

  ///
  DfcVector<int> getDimensions(void) const;
  /**
  {\large Purpose}\\
  Returns the dimensions of the thing we are dealing with. Implements part of the interface inherited from DfcNumericalInterface

  {\large Returns}\\
  The dimensions of the structure. Rather than raise an exception if something non-numeric is being accessed (ie a KeywordStructure, or a Column of Frames, the Dimensions returned will be zero.
   */

  ///
  DFC_REAL_64 getAsReal64(
  ///
  const DfcVector<int> &position
  /**
  The position of the data element that we require.
  */
  ) const;
  // throw(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Obtain a data value as double. Implements the interface inherited from DfcNumericalInterface.
  
  {\large Returns}\\
  A double value.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DFC_INT_2C_64 getAsInt2C_64(
  ///
  const DfcVector<int> &position
  /**
  The position of the data element that we require.
  */
  ) const;
  // throw(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Obtain a data value as int. Implements method from DfcNumericalInterface.
  
  {\large Returns}\\
  An int value.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void setAsReal64(
  ///
  const DfcVector<int> &position, 
  /**
  The position of the data element that we wish to set.
  */
  ///
  DFC_REAL_64 newVal
  );
  // throw(DfcTypeExc, DfcBoundsExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Set a data value as double. Implements method from DfcNumericalInterface.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void setAsInt2C_64(
  ///
  const DfcVector<int> &position, 
  /**
  The position of the data element that we wish to set. Implements a method inherited from DfcNumericalInterface.
  */
  ///
  DFC_INT_2C_64 newVal
  );
  // throw(DfcBoundsExc, DfcTypeExc, DfcDimensionsExc);
  /** {\large Purpose} \\
  Set a data value as an int.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - Could not convert the value to an int, for example maybe we're dealing with a Column of Coordinate Systems.
  \item DfcBoundsExc - the supplied positionVector specified a point outside the bounds of the container object we're using eg we're looking at a column of 30 elements and tried to access element number 50.
  \item DfcDimensionsExc - the position vector had the wrong number of elements in it to specify a point. For example, we're examining a Column and supplied a position Vector that did not have exactly one element in it.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  void setAlias(const DFC_STRING &newAlias);

  ///
  DFC_STRING getAlias(void) const;

  ///
  DfcKeyword *operator()(
  ///
  int position
  /**
  A string for a Keyword to search for in the current Keyword.  
   */
  ) const;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct("instruments")("atmosphere")("seeing").report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A pointer to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  virtual ~DfcKeyword(void);  
protected:
  ///
  void setParent(DfcKeywordInterface *newParent);

  ///
  DfcKeyword *getSafeInterface(void);
  // throw(void)
  /**
  {\large Purpose}\\
  Obtain a proxy object for this object. \\
  As this object is a proxy itself a pointer to this object is returned.
 
  {\large Returns}\\
  A pointer to a DfcKeyword object
  */

  ///
  DfcKeywordInterface *stripProxies(void);
  // throw(void)
  /**
  {\large Purpose}\\
  obtain a pointer to the actual data that a proxy is using.
 
  {\large Returns}\\
  A pointer to a DfcKeyword object
  */

  ///
  void proxyIsDead(void);
  // throw();
  /**
  {\large Purpose}\\
  Informs this object that its proxy object has been deleted. This allows it to make sure there are no dangling pointers. \\
  This method cannot be legitimately called as the proxy has no proxies. Therefore it does nothing but is provided to make this class concrete.
   */

private:
  DfcKeywordInterface *realOne;
};

#endif // DFC_KEYWORD_H
