//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.1 $
//$Log: DfcKeywordInterface.h,v $
//Revision 1.1  2004/09/06 14:35:22  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:02  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:48  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:53  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_KEYWORD_INTERFACE_H
#define DFC_KEYWORD_INTERFACE_H

// includes from this library
#include "DfcExceptions.h"

//
// I mention DfcKeyword (derived from this class) somewhere in the interface so
// it's good practice to provide a forward declaration even though g++ doesn't
// seem to require it.
//
class DfcKeyword;

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

/// abstract class
class DfcKeywordInterface {  // for VLT derives from DfcNumericalInterface
// class DfcKeywordInterface: public DfcNumericalInterface {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcKeywordInterface.h,v 1.1 2004/09/06 14:35:22 cguirao Exp $ } \\

  {\large DESCRIPTION} \\
  Factors out common interface so that the proxy and the KeywordData classes are forced to have the same interfaces. This is an abstract class and instances of it cannot be created.

  {\large EXAMPLES} \\
  See DfcKeyword for useful examples.

*/

public:
  ///
  typedef char* TYPE;

  //
  // typing scheme from M Wiedmer
  // Type is a char *. Therefore comparisons on type are performed
  // on pointers so its quick. At the same time it's easy to obtain
  // a descriptive account of the type rather than some numerical value
  // as we would with an enum.
  // Clearly now, as this is based on pointers the persistence mechanisms
  // cannot use this scheme. However, this is OK as that can be handled
  // completely by the classes themselves at the leaf level.
  //

  ///
  static const TYPE REAL_32;
  ///
  static const TYPE REAL_64;
  ///
  static const TYPE INT_U8;
  ///
  static const TYPE INT_U16;
  ///
  static const TYPE INT_S16;
  ///
  static const TYPE INT_2C_32;
  ///
  static const TYPE INT_2C_64;
  ///
  static const TYPE CHAR;
  ///
  static const TYPE STRING;
  ///
  static const TYPE FRAME_REF;
  ///
  static const TYPE TABLE_REF;
  ///
  static const TYPE TIME;
  ///
  static const TYPE TARGET;
  ///
  static const TYPE ENUM;
  ///
  static const TYPE COORD_SYS;
  ///
  static const TYPE STRUCTURE;

  /// pure virtual
  virtual int getNumElements(void) const = 0;
  // throw();
  /**
  {\large Purpose} \\
  Returns the number of data elements in the Keyword. 

  {\large Returns}\\
  the number of data elements.
  */

  /// pure virtual
  virtual int getNumKeywords(void) const = 0;
  // throw();
  /** {\large Purpose} \\
  Returns the number of Keywords that this Keyword contains. 

  {\large Returns}\\
  the number of keywords.

  */
  
  /// pure virtual
  virtual DFC_STRING getName(void) const = 0;
  // throw();
  /**
  {\large Purpose}\\
  Returns the name of this Keyword.
   */

    /// pure virtual
  virtual DFC_STRING getDescription(void) const = 0;
  // throw();
  /**
  {\large Purpose}\\
  Returns the name of this Keyword.
   */

  /// pure virtual
  virtual void setAlias(const DFC_STRING &nerwAlias) = 0;

  /// pure virtual
  virtual DFC_STRING getAlias(void) const = 0;

  /// pure virtual
  virtual void setDescription(
  ///
  const DFC_STRING &newDesc
  /**
  A new descriptive text string for the Keyword.
   */
  ) = 0;
  // throw();
  /**
  {\large Purpose}\\
  Set a descriptive text field in the Keyword. This forms part of the report produced by the report() method.
  */

  /// pure virtual 
  virtual void setName(
  ///
  const DFC_STRING &newName
  /**
  The new name for the Keyword.
   */
  ) = 0;
  // throw(DfcAlreadyExistsExc);
  /** {\large Purpose} \\
  Set the name of the Keyword. Each Keyword in a KeywordStructure must have a unique name, this is enforced.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - the new name for the Keyword already exists within the containing Keyword which is not allowed. The attempt to set the name has failed and the name remains as it was.
  \end{itemize}

  {\large Arguments}\\
  */

  /// pure virtual
  virtual DfcKeywordInterface::TYPE getType(void) const = 0;
  // throw(); 
  /** {/large Purpose} \\
  An indication of the type of the Keyword. This is used while the standard C++ RTTI mechanisms are still in their infancy and may, or may not, become redundant later. The primary purpose of the type field is to allow persistency, not so that users can query it and perform switch statements on the type. Because of this it may be hidden from the public interface later.
  */

  /// pure virtual
  virtual DFC_STRING report(void) const = 0;
  // throw();
  /** {\large Purpose} \\
  Output some information about this Keyword, probably it's name, type, description and stuff like that.

  {\large Returns}\\
  DFC_STRING containing the report. 
   */


  /// pure virtual
  virtual DfcKeywordInterface *clone(void) const = 0;
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Make a copy of this object. Altering the copy will not affect this object in any way. (technical - we make a deep copy of this object.)  

  {\large Returns}\\
  A pointer to the copy of this object. The responsibility for the memory management of this copy now lies with the method invoker.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  /// pure virtual
  virtual void addCopyOfKeyword(
  /// 
  DfcKeywordInterface &newKeyword
  /**
  A reference to the Keyword to be added into the current Keyword. This Keyword will be copied into the KeywordStructure so the responsibility for the deletion of this argument still lies with the method invoker. Likewise, further changes of this argument will not result in the newly added Keyword being changed. 
  */
  ) = 0;
  // throw(DfcAlreadyExistsExc, DfcTypeExc);
  /** {\large Purpose} \\
  Add a new Keyword (Structure or Data) to the current Keyword.
 
  {\large Preconditions}\\
  The current Keword should really be a KeywordStructure if this method is to succeed.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item DfcAlreadyExistsExc - The name already exists in the current Keyword. Names should be unique in any given KeywordStructure.
  \item DfcTypeExc - the Keyword that we're attempting to add the Keyuword into does not contain Keywords !?!
  \end{itemize}

  {\large Arguments}\\
  */

  /// pure virtual
  virtual void deleteKeyword(
  ///
  const DFC_STRING &target
  /**
  The keyword to find and remove from the current Keyword.
   */
  ) = 0;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find and remove the specified Keyword from the current Keyword.
 
  {\large Preconditions}\\
  the current Keyword should be a KeywordStructure object.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExist - the requested Keyword for removal was not found in the current Keyword (it may be that this Keyword does not contain other Keywords).
  \end{itemize}

  {\large Arguments}\\
  */

  ///pure virtual
  virtual DfcKeywordInterface *operator()(
  ///
  const DFC_STRING &target
  /**
  A string for a Keyword to search for in the current Keyword.  
   */
  ) const = 0;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct("instruments")("atmosphere")("seeing").report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A pointer to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */

  ///pure virtual
  virtual DfcKeywordInterface *operator()(
  ///
  int position
  /**
  The element's index. 
   */
  ) const = 0;
  // throw(DfcDoesntExistExc);
  /** {\large Purpose} \\
  Find a Keyword in the current Keyword. This allows us to navigate the Keyword object hierarchy like traditional C nested structs. For example:\\
  DfcKeyword topStruct; \\
  DfcKeyword seeingInfo = topStruct(1)(1)(3).report(); \\
  \\
  This method is provided as a shorthand way of navigating the hierarchy rather than using iterators or constant statements of find() or some such method.

  {\large Returns}\\
  A reference to the requested Keyword if it exists. 

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcDoesntExistExc - one of the Keywords in the path does not exist.
  \end{itemize}

  {\large Arguments}\\
  */


  /// pure virtual
  virtual DfcKeywordInterface *getParent(void) const = 0;
  // throw()
  /**
  {/large Purpose} \\
  Discover the parent Kewyword of this Keyword. This method is provided with the intent of finding out where in the hierarchy of Keywords one is, not so that the hierarchy can be navigated backwards. Hence the const status of the Keyword.

  {/large Returns} \\
  const pointer to the parent object if this Keyword has a parent, otherwise to Null.
  */

  /// pure virtual
  virtual DFC_REAL_32 getAsReal32(
  ///
  int whichElement = 0
  /**
  Each Keyword can contain several items of data (of the same type). This field specifies which one is being requested.
   */
  ) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);
  /** {\large Purpose} \\
  Obtain data from the Keyword as a 32 bit real value.

  {\large Preconditions}
  0 <= whichElement < this::getNumElements()

  {\large Returns}\\
  A 32 bit real.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcTypeExc - the data cannot be obtained as a 32 bit real. For example, this Keyword contains Time objects or other Keywords.
  \item DfcBoundsExc - the requested element is not a valid one.
  \end{itemize}

  {\large Arguments}\\
  */

  /// pure virtual
  virtual DFC_REAL_64 getAsReal64(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DFC_INT_UNSIGN_8 getAsIntUnsign8(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DFC_INT_UNSIGN_16 getAsIntUnsign16(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DFC_INT_SIGN_16 getAsIntSign16(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DFC_INT_2C_32 getAsInt2C_32(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  ///pure virtual
  virtual DFC_INT_2C_64 getAsInt2C_64(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual char getAsChar(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DFC_STRING getAsString(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DfcCoordSys getAsCoordSys(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DfcTime getAsTime(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DfcFrame getAsFrameRef(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DfcTable getAsTableRef(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DfcEnumerated getAsEnumerated(int whichElement = 0) const = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DfcTarget getAsTarget(int whichElement = 0) const = 0;  
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsReal32(DFC_REAL_32 newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsReal64(DFC_REAL_64 newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsIntUnsign8(DFC_INT_UNSIGN_8 newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsIntUnsign16(DFC_INT_UNSIGN_16 newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsIntSign16(DFC_INT_SIGN_16 newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsInt2C_32(DFC_INT_2C_32 newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsInt2C_64(DFC_INT_2C_64 newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsChar(char newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsString(const DFC_STRING &newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsCoordSys(const DfcCoordSysInterface &newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsTime(const DfcTimeInterface &newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsFrameRef(DfcFrameInterface &newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsTableRef(DfcTableInterface &newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsEnumerated(const DfcEnumeratedInterface &newValue, int whichElement = 0) = 0;
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual void setAsTarget(const DfcTargetInterface &newValue, int whichElement = 0) = 0;  
  // throw(DfcTypeExc, DfcBoundsExc);

  /// pure virtual
  virtual DFC_BOOL operator==(const DfcKeywordInterface &other) const = 0;
  /** {\large Purpose} \\
  Equality operator (associated operators must be defined later. Only performs comparisons on the basis of the name field. This is how the objects are to be uniquely identified.
  */

  /// pure virtual
  virtual DfcKeywordInterface &operator=(const DfcKeywordInterface &other) = 0;
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */


  ///
  DFC_BOOL constructedOK(void) const;

  ///
  DFC_STRING getReason(void) const;

  ///
  virtual ~DfcKeywordInterface(void);  
protected:
  ///
  friend class DfcKeyword;
  friend class DfcKeywordStructure;
  /**
  Never happy about using friends, however it is better to do this than to 
  make these classes public.
   */

  /// pure virtual
  virtual void setParent(DfcKeywordInterface *newParent) = 0;

  /// pure virtual
  virtual DfcKeyword *getSafeInterface(void) = 0;
  // throw(void)
  /**
  {\large Purpose}\\
  Return a pointer to the actual object that this object refers to, if it is a proxy.
 
  {\large Returns}\\
  A pointer to a DfcKeyword object
  */

  ///
  DfcKeywordInterface(void);

  ///
  DfcKeywordInterface(const DfcKeywordInterface &other);

  /// pure virtual
  virtual DfcKeywordInterface *stripProxies(void) = 0;
  // throw();
  /** 
  {\large Purpose} \\
  obtain a pointer to the actual data that a proxy is using.
  
  */


  /// pure virtual
  virtual void proxyIsDead(void) = 0;
  // throw();
  /**
  {\large Purpose}\\
  Informs this object that its proxy object has been deleted. This allows it to make sure there are no dangling pointers.
   */

  ///
  void constructionFailed(const DFC_STRING &reason);
  // throw();
  /**
  {\large Purpose}\\
  Set some status vars that record the fact that the object was not constructed properly. This is a really poor way of checking the status of objects but so far I have had two major problems with throwing exceptions in constructors - one "internal compiler error" during compilation, and upon throwing an exception once g++ lost track of the vtable and started calling random methods.
   */

private:

  DFC_BOOL constructionSucceeded;
  /**
  Records whether the construction of the object succeded or not. This is
  necessary because g++ really screws up on exceptions. So far I have only had
  problems with throwing them in constructors so I intend to use them elsewhere
  unless problems show up.
   */

  DFC_STRING failureReason;
  /**
  Reason why the construction of this object failed.
   */
};

#endif // DFC_KEYWORD_INTERFACE_H







