//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Alisdair Manning - ESO-DMD-DPG>
//SUBSYSTEM: <df-core>
//
//$Revision: 1.1 $
//$Log: DfcStringKeyword.h,v $
//Revision 1.1  2004/09/06 14:35:23  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/09/02 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/24 13:12:16  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:05:07  dataflow
//Compatible with g++ 3.0.4 CG
//
//Revision 1.2  1998/06/30 08:57:56  dataflow
//Removing $NAME
//
//Revision 1.1  1996/12/20 11:40:57  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/10 07:48:21  amanning
//first release - prior to first design review
//
//-----------------------------------------------------------------------------

#ifndef DFC_STRING_KEYWORD_H
#define DFC_STRING_KEYWORD_H

// includes from this library
#include "DfcKeywordData.h"

// includes from other ESO libraries
// includes from other libraries
// system includes

// Exception classes used in this class

///
class DfcStringKeyword: public DfcKeywordData<DFC_STRING> {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: df-core} \\
  {\footnotesize $Id: DfcStringKeyword.h,v 1.1 2004/09/06 14:35:23 cguirao Exp $ } \\

  {\large DESCRIPTION} \\
  Keyword that holds a number of Stringinates. The Stringinates can be any class derived from the DfcString class.
 
  {\large DESIGN DECISIONS}\\
  Now, this is a really tricky one. We have a bunch of DfcCordinateSystems. Each derived class can have different methods. Even though this is a container class we don't want to expose the contained objects directly to the user. Then it would be possible for them to delete the contents of the elements so the Keyword still thinks it has the data but all it has is a pointer to released memory. Very dangerous. It's not so dangerous in the case of Keywords but it is with Columns, when a Column can actually operate on its contents. Clearly the usage between Keywords and Columns should be as similar as possible. \\
  So we can't store a pointer or a reference and pass this back to the user directly. Thsi means copying data in and out. I have already established that DfcStringinate doesn't have to have a fat interface, we can usual virtual methods to easily convert between one type of cordinate system and another. However, this relies upon a pointer being passed which isn't safe. Therefore we have to introduce another proxy object which hides a pointer to the base class and provides basic information. \\
  Final issue is whether to store pointers to the objects in the Keyword, or actual objects themselves. If the containing objects are to perform operation on the contained objects then it is better that each object in the container is identical and hence has the same interface. However, for Columns and Keywords this isn't an issue. If we store just pointers then it means that we can have Columns containing arbitrary types of Stringtems. ScalarColumns cannot contain arbitrary types of Scalars mixed together (because they're not classes). I could make Stringtem Keywords consistent with this approach, however, I'm concerned that this would lead to an explosion in template generated source code so I'm going to sacrafice a little bit of consistency and say that arbitrary types of Stringtems can be stored in a single Keyword. (It could be made consistent again by ONLY storing doubles in ScalarKeywords).
*/

public:
  ///
  DfcStringKeyword(
  ///
  const DfcKeywordInterface &other
  );
  // throw(DfcMemoryException);
  /** {\large Purpose} \\
  copy constructor

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryException - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcStringKeyword(
  ///
  const DfcStringKeyword &other
  /**
  The Keyword that is to be copied. This Keyword is in no way referenced by the new Keyword afterwards.
  */
  );
  // throw(DfcMemoryException);
  /** {\large Purpose} \\
  copy constructor

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryException - for some bizarre reason, there isn't enough memory to create a copy of the Keyword.
  \end{itemize}

  {\large Arguments}\\
  */

  ///
  DfcStringKeyword(
  ///
  const DFC_STRING &newName, 
  /**
  A new name for the Keyword.
   */  
  ///
  const DFC_STRING &alias,
  /**
  An alias for the Keyword.
   */
  ///
  const DFC_STRING &newDescription,
  /**
  Arbitrary text string to store with the Keyword.
  */
  ///
  int numElements = 1
  );
  // throw(DfcMemoryExc); 
  /**
  {\large Purpose} \\
  Create a new Keyword capable of storing the specified amount of data.
   */

  ///
  DFC_STRING getAsString(int whichElement = 0) const;
  // throw(DfcTypeException, DfcBoundsException);

  ///
  void setAsString(const DFC_STRING &newValue, int whichElement = 0);
  // throw(DfcTypeException, DfcBoundsException);
 

  ///
  DfcKeywordInterface::TYPE getType(void) const;
  // throw(); 
  /** {/large Purpose} \\
  An indication of the type of the Keyword. This is used while the standard C++ RTTI mechanisms are still in their infancy and may, or may not, become redundant later. The primary purpose of the type field is to allow persistency, not so that users can query it and perform switch statements on the type. Because of this it may be hidden from the public interface later.
  */

  
  ///
  DfcStringKeyword *clone(void) const;
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Make a copy of this object. Altering the copy will not affect this object in any way. (technical - we make a deep copy of this object.)  

  {\large Returns}\\
  A pointer to the copy of this object. The responsibility for the memory management of this copy now lies with the method invoker.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */


  /// 
  DfcStringKeyword &operator=(const DfcKeywordInterface &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only. All contents will not be copied.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */


  /// 
  DfcStringKeyword &operator=(const DfcStringKeyword &other);
  // throw(DfcMemoryExc, DfcAlreadyOpenExc);
  /** {\large Purpose} \\
  Perform a shallow copy of the Keyword. All data will be shared between the original and the new version as the copying is a shallow copy only. All contents will not be copied.

  {\large Returns}\\
  Reference to this object to allow chaining of assignment operators.

  {\large Exceptions}\\
  \begin{itemize}
  \item DfcMemoryExc - there was not enough memory on the local machine to perform the copy succesfully.
  \item DfcAlreadyOpenExc - the original is opened for update. There can only be one copy of a Keyword open for update at any one time, therefore the clone() operation has to fail.
  \end{itemize}

  */

  ///
  virtual ~DfcStringKeyword(void);  
protected:
  ///
  DfcStringKeyword(void);
  /**
  {\large Purpose} \\
  Hide the default constructor. I don't want anybody creating a Keyword with so little data.
  */

private:

};

#endif // DFC_STRING_KEYWORD_H

//
// eof
//





