package org.eso.ohs.phase2.orang;

import java.util.Date;
import java.util.Vector;

import org.eso.ohs.core.gui.widgets.ProgressModel;
import org.eso.ohs.core.gui.widgets.ProgressModelDefault;
import org.eso.ohs.core.gui.widgets.ProgressModelListener;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.Seeing;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.dfs.WeatherConstraint;
import org.eso.ohs.dfs.WindInfo;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.phase2.visibility.VisibilityCalcUtils;


import com.mhuss.AstroLib.AstroOps;

/**
 * This class filter ObDataContainer objects (light-weight OBs) 
 * according to weather constraints
 * 
 * @author pnunes 24.08.2005
 *
 */
public class ObWeatherFilter implements ProgressModel {

    private static final String WIND_ANGLE = "ORANG.WIND.CONSTRAINT.ANGLE";
    /**
     * Tolerance constant to compare double values 
     */
    private static final double TOLERANCE = 0.0001;

    private ObDataContainer[] obDataArray_;

    /*
     *  ProgressModel instance
     */
    private ProgressModelDefault progressModel;
    
    
    
    public ObWeatherFilter(ObDataContainer[] obList) {
        setObDataArray(obList);
        
    }

    public ObWeatherFilter() {
        obDataArray_ = new ObDataContainer[0];
        progressModel = new ProgressModelDefault(1,1,"Filtering by weather...");
    }

    public void setObDataArray(ObDataContainer[] obList) {
        obDataArray_ = obList;
        
        int max = 1;
        if (obList.length >0 )
        	max = obList.length;        	
        progressModel = new ProgressModelDefault(1,max,"Filtering by weather...");
    }

    /**
     * Filters the OBs given an ConstraintSet business object
     * @param weatherConditions
     * @return
     */
    public ObDataContainer[] filterObArray(ConstraintSet weatherConditions,Date startDate) {
        Seeing minSeeing = new Seeing(weatherConditions.getSeeing());
        Seeing maxSeeing = new Seeing(weatherConditions.getMaxSeeing());
          WindInfo wind = new WindInfo(weatherConditions.getWindAngle());
            SkyTransparency sky = new SkyTransparency(weatherConditions
                .getCurrDbSkyTransparency());
        WeatherConstraint currentConstraint = new WeatherConstraint(minSeeing,maxSeeing,
                sky,wind);
        
        setValue(1);
        ObDataContainer[] obArray = filterObArray(currentConstraint,startDate);

        return obArray;
    }

    
    
    /**
     * Filters the OBs given an WeatherConstraint  object
     * 
     * @param currentConstraint
     * @return
     */
    public ObDataContainer[] filterObArray(WeatherConstraint currentConstraint,Date startDate) {
        Vector<ObDataContainer> obDataVector = new Vector<ObDataContainer>();

        for (int i = 0; i < obDataArray_.length; i++) {
            ObDataContainer currentOb = obDataArray_[i];

            // updates the progress model
            setValue(getValue()+1);
            
            
            boolean seeingOk = true;
            if (currentConstraint.isSeeingEnable()) {
                
                if ((currentConstraint.getMinSeeing().doubleValue() -TOLERANCE < currentOb
                        .getObInfo()
                        .getSeeing())&& 
                        (currentConstraint.getMaxSeeing().doubleValue() +TOLERANCE > currentOb
                                .getObInfo()
                                .getSeeing())) {
                    seeingOk = true;
                } else {
                    seeingOk = false;
                }
            } else {
                seeingOk = true;
            }

            boolean skyOk = true;
            if (currentConstraint.isSkyEnable()) {
                String skyString = currentOb.getObInfo().getSkyTransparency();
              //  String currentSkyDB = SkyTransparency.toDbSkyTransparency(skyString);
              
                if (currentConstraint.getSky().get().compareTo(skyString) <= 0) {
                    skyOk = true;
                } else {
                    skyOk = false;
                }
            } else {
                skyOk = true;
            }
            boolean windOk = true;
            if (currentConstraint.isWindEnable()) {
                double currentAzimuthAngle = getAzimuth(currentOb,startDate);
                double angularDistance = getWindDirectionDistance(currentAzimuthAngle, currentConstraint.getWind());
               currentOb.setWindDistance(angularDistance);
                if (isWindAngleOk (angularDistance))    {
                    windOk = true;
                } else {
                    windOk = false;
                }
            } else {
                currentOb.clearWindDistance();
                windOk = true;
            }
            if (windOk == true && skyOk == true && seeingOk == true) {
                obDataVector.addElement(currentOb);
            }
        }
        return obDataVector.toArray(new ObDataContainer[obDataVector.size()]);
    }

	private boolean isWindAngleOk(double angularDistance) {
        
        boolean result = false;
        int lowerLimit = AppConfig.getAppConfig().getInteger(
                WIND_ANGLE,WindInfo.LOWER_AZ_LIMIT);
        int upperLimit = WindInfo.ANGLE_CONVERSION - lowerLimit;
         if (angularDistance > lowerLimit && 
                angularDistance < upperLimit) {
            result = true;
        } else {
            result = false;
        }
        return result;
    }

    /**
     * @param currentAzimuthAngle
     * @param wind
     * @return double angular distance (absolute value)
     */
    private double getWindDirectionDistance(double currentAzimuthAngle, WindInfo wind) {
        int windAngle = wind.getWindAngle();
        
        if (windAngle < 0) {
            windAngle = WindInfo.ANGLE_CONVERSION + windAngle;
        }  
       
        double azDegree = Math.toDegrees(currentAzimuthAngle);
        if (azDegree < 0) {
            azDegree = WindInfo.ANGLE_CONVERSION + azDegree;
        } 
        double angularDistance = Math.abs(azDegree - windAngle);
        return angularDistance;
    }

    private double getAzimuth(ObDataContainer ob,Date startDate) {
        
        double dbRA = ob.getObInfo().getRa();
        double dbDEC = ob.getObInfo().getDec();
        /// converts
        double targetRA = VisibilityCalcUtils.millisecsToRadians(dbRA);
        double targetDEC = VisibilityCalcUtils.milliarcsecsToRadians(dbDEC);
//      calculate local sidereal time
        double jdate = VisibilityCalcUtils.getJulianDate(startDate,0,0);
        double gmst = AstroOps.greenwichSiderealTime(jdate);
        gmst = AstroOps.normalizeRadians(gmst);
        double longitude = AppConfig.getAppConfig().getLongitude();
        double lat = AppConfig.getAppConfig().getLatitude();
        Double lst = new Double(gmst - longitude);     
    
          double azimuth = VisibilityCalcUtils.getAzimuth(targetRA, targetDEC, lat, lst.doubleValue()); 
          return azimuth;
    }

    public int getMaximum() {
		return(progressModel.getMaximum());
	}

	public int getMinimum() {
		return progressModel.getMinimum();
	}

	public int getValue() {
		return progressModel.getMaximum();
	}

	public void setMaximum(int max) {
		progressModel.setMaximum(max);
		
	}

	public void setMinimum(int min) {
		progressModel.setMinimum(min);
		
	}

	public void setValue(int value) {
		progressModel.setValue(value);
		
	}

	public String getMsg() {
		return(progressModel.getMsg());
	}

	public void setMsg(String msg) {
		progressModel.setMsg(msg);		
	}

	public void addChangeListener(ProgressModelListener l) {
		progressModel.addChangeListener(l);
		
	}

	public void removeChangeListener(ProgressModelListener l) {
		progressModel.removeChangeListener(l);
		
	}
}
