package org.eso.ohs.phase2.visibility;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.apache.log4j.Logger;
import org.eso.ohs.dfs.TimeInterval;


/**
 * 
 * Created on Jul 25, 2005
 * @author pnunes 
 *
 * This class implements some utility methods used in visibilities computation
 * 
 */
public class VisibilityCalcUtils {

	protected static Logger stdlog_ = Logger.getLogger(VisibilityCalcUtils.class);
	
	
	/**
	 * Constants to convert milliarc seconds to  radians
	 */
	public static final double MILLARC_HRMMSS = 7.272205216643E-8;
	public static final double MILLARC_MMSS = 206265.0;
	
	
	public static final int MILLISECS_PER_SEC = 1000;
	
	// paranal coordinates
	public static double PLAT = -0.429802; //(radians)
	public static double PLONG = 1.228768; //(radians)
	public static final int DURATION = 12;
	
	
	
	/**
	 *  Initialise the lognitude and latitude from AppConfig
	 */
	public VisibilityCalcUtils() {}	


	/**
	 * Initialize the AirMass PCF
	 */
	public void initAirMassPCF(double airConstraint, PCF airmassPCF) {
	    double[] values = new double[6];
	    double step = 1.0;
	    for (int i = 0; i < values.length; i++) {
	        values[i] = step;
	        step = step + 0.5;
	    }
	    AirMassFilter airMassFilter =
	        new AirMassFilter(new double[0], airConstraint);
	    airmassPCF = new PCFPlotter(AirMass.NOT_VIS, airMassFilter);
	}

	/**
	 * getDateZeroHours get Julian date at 0 hours 0 minutes and 0 secs
	 * @param startdate Date 
	 * @return Julian date double 
	 * see JulianDate#getJDC(Date)
	 */
	public static double getDateZeroHours(Date startDate) {
	    double jdate = JulianDate.getJDC(startDate);
	    return (jdate);
	}

    /**
     * Calculate the duration of the plot.
     * Subtract the received end date/time from the start date/time
     * divide the difference by the number of milliseconds in a day.  
     * Determine if the this is more or less than one day and 
     * return the duration.
     *
     * @param startDate Date
     * @param endDate Date
     * @return duration integer
     */
    
    public static int getDuration(Date startDate, Date endDate) {
        double timeBetween = (endDate.getTime() - startDate.getTime());
        timeBetween = timeBetween / (1000. * 60. * 60.);
        
        return (int) (timeBetween );
        
        
//      if (timeBetween < 1.)
//          return (int) (timeBetween * 24.);
//      else {
//          return (int)
//          ((Math.floor(timeBetween) * 24.)
//                  + ((Math.floor(timeBetween) - timeBetween) * 24.));
//      }
    }
    
    /**
     * Calculate the number of steps for the visibility computation, given start, end time
     *  and number of minutes in one step.
     *  If the DateRange is wrong or the interval is bigger than the range an IllegalArgumentException
     *  will be raised.
     * 
     * @param startDate Date
     * @param endDate Date
     * @param interval int number of minute per step.
     * @return numberOfSteps integer the minimun is 1;
     */
    
    public static int getNumberOfSteps(Date startDate, Date endDate,int interval) {
        long differenceInMillisecs = (endDate.getTime() - startDate.getTime());
        long differenceInMinutes = differenceInMillisecs / (1000 * 60);
        if (startDate.after(endDate)) throw new IllegalArgumentException("Start Date is after End Date");
        if (differenceInMinutes<interval)    throw new IllegalArgumentException("Difference between start date and and date is lower than the interval");
        int numberOfSteps = (int) Math.floor(differenceInMinutes / interval);
        return numberOfSteps;
        
    }
    

	/**
	 * getIntervalEndTime get start date + interval
	 * @param startdate Date 
	 * @param interval integer
	 * see JulianDate#getJDC(Date, int)
	 */
	public static Date getIntervalEndTime(Date startDate, int interval) {
		//stdlog_.debug("getIntervalEndTime( "+startDate+" , "+interval+" )");
	    Date jdate = JulianDate.getJDC(startDate, interval);
	    return (jdate);
	}

	/**
	 * getJulianDate get Julian date + interval
	 * @param startdate Date
	 * @param interval integer time of each plot interval
	 * @param ctr integer the position the loop
	 * @return Julian date double 
	 * see JulianDate#getJDC(Date, int, int)
	 */
	public static double getJulianDate(Date startDate, int interval, int ctr) {
	    double jdate = JulianDate.getJDC(startDate, interval, ctr);
	    return (jdate);
	}

	/**
	 * getTimeIntervalValue - get the Time Interval Value 
	 * If the time interval has not been defined in the OB
	 * any time period is satisfactory set the returned value to 1. 
	 * Else if the start and end time does not fall within
	 * this interval time (precision) set returned value to 0. 
	 * @param ObservationBlock
	 * @param julian Date double
	 * @return time interval value
	 */
	public static double getTimeIntervalValue(TimeInterval[] timeInts, Date date) {
	    
	    double timeValue = 1.;
	
	    
	    
	    if (timeInts != null && timeInts.length != 0) {
	        for (int i = 0; i < timeInts.length; i++) {
	            Date startTime =
	                new Date(timeInts[i].getStartTime() * MILLISECS_PER_SEC);
	            Date endTime =
	                new Date(timeInts[i].getEndTime() * MILLISECS_PER_SEC);
	            /*
	            stdlog_.info("visiplot.VisiPlot.getTimeIntervalValue start: "+jstart); 
	            stdlog_.info("visiplot.VisiPlot.getTimeIntervalValue   end: "+jend);
	            stdlog_.info("visiplot.VisiPlot.getTimeIntervalValue jdate: "+jdate); 
	            */
	            if ( date.after(startTime) && date.before(endTime)  ) {	            	           
	                timeValue = 1.;
	                return (timeValue);
	            } else
	                timeValue = 0.;
	        }
	    }
	    return (timeValue);
	}
	
	
	/**
	 * Converts a Julian date to a modified Julian Date
	 * @param jDate
	 * @return modified julian date
	 */
	public double jDateToModifiedJDate(double jDate) {
		return(jDate - 2400000.5);
	}
	
	/**
	 * Converts a Modified Julian date to a Julian Date
	 * @param jDate
	 * @return julian date
	 */
	public double modifiedJDateToJDate(double jDate) {
		return(jDate + 2400000.5);
	}
	
	
	
	/**
	 * @param dec
	 * @param obsLat
	 * @param ha
	 * @return double
	 */
	public static double azimuthFromHa(double dec, double obsLat, double ha) {
	    double az = Math.atan2(-Math.sin(ha) * Math.cos(dec), Math.cos(obsLat)
	            * Math.sin(dec) - Math.sin(obsLat) * Math.cos(dec)
	            * Math.cos(ha));
	    return az;
	}


	/**
	 * get the azimuth in radians. (from -180 to 180). 0 is North
	 * @param ra
	 * @param dec 
	 * @param obsLat latitude
	 * @param lst local sid time
	 * @return the azimuth
	 */
	public static  double getAzimuth(double ra, double dec, double obsLat, double lst) {
	    double ha = getHourAngle(ra, lst);    
	    // azimuth angle (radians)
	    double az = VisibilityCalcUtils.azimuthFromHa(dec, obsLat, ha);
	    return az;
	}


	/**
	 *
	 * Returns the target's elevation, given declination, latitude and hour angle
	 * 
	 * @param targetRA
	 * @param targetDec
	 * @param obsLat
	 * @param lst
	 * @return
	 */
	public static double getElevation(double targetRA, double targetDec, double obsLat, double lst) {
	    //       hour angle
	    double ha = getHourAngle(targetRA, lst);    
	    
	    
	    
	    // elevation (radians)
	    double el = Math.asin(Math.sin(obsLat) * Math.sin(targetDec)
	            + Math.cos(obsLat) * Math.cos(targetDec) * Math.cos(ha));
	    
	    //stdlog_.debug(targetRA + "," + targetDec + "," + obsLat + "," + lst + "," + el);
	    return el;
	}


	/**
	 * @param targetRA
	 * @param lst
	 * @return double
	 */
	public static double getHourAngle(double targetRA, double lst) {
	    double ha = lst - targetRA;
	    return ha;
	}


	/**
	 * Converts angle from milliarcsecs to radians
	 * 
	 * @param angle
	 * @return
	 */
	public static double milliarcsecsToRadians(double angle) {
		double result;
		result = ( angle / VisibilityCalcUtils.MILLARC_MMSS) / VisibilityCalcUtils.MILLISECS_PER_SEC;
		return result;
	}
	
	/**
	 * Converts angle from radians to milliarcsecs
	 * 
	 * @param angle
	 * @return
	 */
	public static double radiansToMilliarcsecs(double angle) {
		double result;
		double degrees = Math.toDegrees(angle);
		result = ( degrees * 60 * 60 * 1000);
		return result;
	}


	/**
	 * Converts angle from millisecs to radians
	 * 
	 * @param angle
	 * @return
	 */
	public static double millisecsToRadians(double angle) {
		double result;
		result = angle * VisibilityCalcUtils.MILLARC_HRMMSS;
		return result;
	}


	/**
	 * Computes the conjunction (logical and) of two double arrays
	 * assuming that both have the same length
	 * 
	 * @param first
	 * @param second
	 * @return double array
	 */
	public static double [] conjunction (double [] first , double [] second ) {
		double [] res = new double[first.length]; 
		for (int i = 0; i < first.length; i++) {
			double f = first[i];
			double s = second[i];
			
			res[i] = f * s;		
		}
		
		
		return(res);
	}
	
	/**
	 * Computes the disjuction (logical or) of two double arrays
	 * assuming that both have the same length
	 * 
	 * @param first
	 * @param second
	 * @return double array
	 */
	public static double [] disjunction (double [] first , double [] second ) {
		double [] res = new double[first.length]; 
		for (int i = 0; i < first.length; i++) {
			double f = first[i];
			double s = second[i];
			
			if ( f + s > 0 )
				res[i] = 1;
			else
				res[i] = 0;
		}
		
		
		return(res);
	}
	
	
	
	/**
	 * Computes the conjunction (logical and) of and array of double arrays
	 * assuming that all have the same length
	 * 
	 * @param array
	 * @return double array
	 */
	public static double [] conjunction (double [][] array ) {
		double [] res = new double[array[0].length]; 
		for (int i = 0; i < array[0].length; i++) {
			
			double andResult = 1;
			
			for (int j = 0; j < array.length; j++) {
				andResult = andResult * array[j][i];
			}
			
			res[i] = andResult;	
		}
		
		return(res);
	}


	/**
	 * Returns a Date incremented by a number of minutes
	 * 
	 * @param startTime
	 * @param startTimeDelay minutes
	 * @return
	 */
	public static Date delayTime(Date startTime, int startTimeDelay) {
		return( new Date(startTime.getTime() + MILLISECS_PER_SEC*60*startTimeDelay) );
		
		
	}
	
	
	/**
	 * Rounds a date to the next "round time", according to a precision given in minutes
	 * 
	 * For example: 2006-03-12T22:14:34 gets rounded to 2006-03-12T22:20:00 for a precision of 10 minutes
	 * 
	 * @param start Start date
	 * @param precision (minutes)
	 * @return
	 */
	public static Date roundDate(Date start, int precision) {		
		Date res = null;		
	
		Calendar cal = new GregorianCalendar();
		cal.setTime(start);
		
		int minutes = cal.get(Calendar.MINUTE);
		
		cal.add(Calendar.MINUTE,precision - (minutes % precision));
		cal.set(Calendar.SECOND,0);
		
		
		res = cal.getTime();
		return(res);
	}
    
         
     	

	/**
	 * invertAirMass set the y-axis of the plot
	 * NOT USED AS OF 26/09/01
	 *
	 */
	/*
	private void invertAirMass() {
	    int pcfLength = airMassPlotter_.getIntervalCount();
	    for (int i = 0; i < pcfLength; i++) {
	        PCFPoint p = airMassPlotter_.getPointAt(i);
	        if (p.getValue() > 3.5)
	            p.setValue(0.0);
	        if (p.getValue() == 3.5)
	            p.setValue(0.0);
	        if (p.getValue() == 3.)
	            p.setValue(0.2);
	        if (p.getValue() == 2.5)
	            p.setValue(0.4);
	        if (p.getValue() == 2.)
	            p.setValue(0.6);
	        if (p.getValue() == 1.5)
	            p.setValue(0.8);
	        if (p.getValue() == 1.)
	            p.setValue(1.0);
	    }
	    //stdlog_.info("\nvisiplot.Visiplot.invertAirMass AirMassPCF:\n"+airMassPlotter_.toString());
	}
	
	*/	 
    
}
