package org.eso.ohs.phase2.visibility;

/*
 * 
 * This class computes visibilities of observation blocks
 * 
 * Created on Jun 7, 2005
 * @author pnunes and ddorigo
 *
 */

import java.util.Date;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ProgressModel;
import org.eso.ohs.core.gui.widgets.ProgressModelDefault;
import org.eso.ohs.core.gui.widgets.ProgressModelListener;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.orang.ObDataContainer;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter.ValueRejectedException;
import org.eso.ohs.phase2.visiplot.TimerLog;

import com.mhuss.AstroLib.AstroOps;
import com.mhuss.AstroLib.Latitude;
import com.mhuss.AstroLib.Longitude;
import com.mhuss.AstroLib.NoInitException;
import com.mhuss.AstroLib.ObsInfo;
import com.mhuss.AstroLib.PlanetData;
import com.mhuss.AstroLib.Planets;



public class VisibilityJCalc implements ProgressModel {
	 
	/**
	 * Visibility data cache 
	 */
	VisibilityCache visibilityCache_;
	
	
	/*
     *  ProgressModel instance
     */
    private ProgressModelDefault progressModel;
	
    
    private boolean cacheEnabled_ = false;
    
	/**
     * Logging instance.
     */
    protected static Logger stdlog_ =
        Logger.getLogger(VisibilityJCalc.class);

    public VisibilityJCalc() {
    	progressModel = new ProgressModelDefault(1,1,"Computing visibilities...");
    	visibilityCache_ = VisibilityCache.getInstance();
    }
    
    
	/*
	 * Calculates the visibility information for an array of OBs for a period of time	 *
	 *   	 
	 * @param obs  OB data container objects
	 * @param interval  (minutes) 
	 * @param startTime
	 * @param endTime
	 * @param observLatitude ( Latitude of the observatory )
	 * @param observLongitude ( Longitude of the observatory ) 
	 * @return array of OBScheduleInfo objects
	 */
	public OBScheduleInfo [] calc(ObDataContainer [] obs , int interval,
			Date startTime, Date endTime, 
			VisicalcOptions options, double observLatitude, double observLongitude ) throws ValueRejectedException  {
	    String[] splitString = {"Sun/moon positions","Sun/moon plot functions","Airmass", "Moon distance","Time intervals","Sidereal Time Intervals", "Plot functions","Conbine constrainst","Zenith Constraint","OBInfo"};
		TimerLog timer = new TimerLog(splitString);
		timer.setCalculate(true);
		
		
		double jDate = 0.0;
		double targetRA = 0.0;
		double targetDEC = 0.0;
		double sunAngle = 0.0;
		double fli = 0.0;
		double moonDist = 0.0;
		double moonEl = 0.0; 
		double moonUp = 0.0 ; 
		double airMass = 0.0;
		double timeInterval = 0.0;
		double lst = 0.0;
		
		
		double moonDistanceTolerance = options.getMoonDistanceTolerance(); 
		double airmassTolerance = options.getAirmassTolerance();
		double fliTolerance = options.getFliTolerance();
		double sunConstraint = options.getSunConstraint();
		double moonConstraint = options.getMoonConstraint();
		
		boolean calcFli = options.calcFli();
		boolean calcMoonDist = options.calcMoonDist();
		boolean calcMoon = options.calcMoon();
		boolean calcSun = options.calcSun();
		boolean calcTimeInterval = options.calcTimeInterval();
		boolean calcSiderealTimeInterval = options.calcSiderealTimeInterval();
		boolean calcZenithConstraint = options.calcZenithInvisibility();
		
		double [] moonElevations;
		double [] moonUps;
		double [] sunAngles;
		double [] flis;
		double [] timeIntervals;
		double [] siderealTimeIntervals;
		double [] airMasses;
		double [] moonDistances;
		double [] meetZenithConstraint;
		double [] execution;
		double [] result ;
		double [] lsts;
		double [] moonRAs;
		double [] moonDecs;
		double [] sunDowns;
		
		// apply the delay
		//startTime = VisibilityCalcUtils.delayTime(startTime,options.getStartTimeDelay());
		
		
		OBScheduleInfo [] resInfo = new OBScheduleInfo[obs.length];
		
		// sets the upper limit of iteration
		int total = VisibilityCalcUtils.getNumberOfSteps(startTime, endTime,interval);
				
		
		moonElevations = new double[total];		
		sunAngles = new double[total];
		moonUps = new double[total];			
		flis = new double[total];
		meetZenithConstraint = new double[total];
		lsts = new double[total];
		moonRAs = new double[total];
		moonDecs = new double [total];
		sunDowns = new double [total];
		
		
		//Iterate through the interval and
		//calculate Moon positions, sun and moon angle 		 	
		double [] julianDates = new double[total];
		
		for (int i = 0; i < total; i++) {
			timer.calcTimer(0);
			
			// Calculate the Julian Date plus the interval time
			jDate = VisibilityCalcUtils.getJulianDate(startTime, interval, i);
			julianDates[i] = jDate;
			
			// local sidereal time
			lst = getLST(jDate,observLongitude);
			lsts[i] = lst;

			// 	sun elevation
			if ( calcSun) {
				sunAngle = getSunValue(jDate,observLongitude,observLatitude,lst);				
			}
			
			// moon calculations
			if (calcMoon) {
				
				// moon positions (assuming caching)
				moonRAs[i] = getMoonRa(jDate, observLongitude, observLatitude);
				moonDecs[i] = getMoonDec(jDate, observLongitude, observLatitude);
				
				// moon elevation
				moonEl = getMoonAngle(jDate, observLongitude, observLatitude);
				moonUp = getMoonVisible(jDate,observLongitude,observLatitude);
				
				if (calcFli) {		
					fli = getFLIValue(jDate,observLongitude,observLatitude);										
				}

			}
			
			sunAngles[i] = sunAngle;
			moonElevations[i] = moonEl;
			moonUps[i] = moonUp;
			flis[i] = fli;	
			timer.calcTimer(0);
	
		}
		
		timer.calcTimer(1);
		
		sunDowns = this.getSunDownBinary(sunAngles,sunConstraint);
		
		timer.calcTimer(1);
				
		// Calculate airmass and moon angle for each Observation Block
		// and store this information on the OBScheduleInfo
		
		// resets the progress model
		
		progressModel.setMinimum(1);
		progressModel.setMaximum(obs.length);
		progressModel.setValue(1);
		
		for (int i = 0; i < obs.length; i++) {
            
            //stdlog_.debug( " Calculate visibility constraint "+i);
         
		    ObDataContainer ob = obs[i];
			
		    double dbRA = ob.getObInfo().getRa();
		    double dbDEC = ob.getObInfo().getDec();
		    
			/// converts
			targetRA = VisibilityCalcUtils.millisecsToRadians(dbRA);
	        targetDEC = VisibilityCalcUtils.milliarcsecsToRadians(dbDEC);
	        
	        //stdlog_.debug("Paranal latitude(degrees): " + Math.toDegrees(observLatitude));
	        //stdlog_.debug("Paranal longitude (degrees): " + Math.toDegrees(observLongitude));
	        
	        //stdlog_.debug("target RA (degrees): " + Math.toDegrees(targetRA));
	        //stdlog_.debug("target DEC (degrees): " + Math.toDegrees(targetDEC));

			airMasses = new double[julianDates.length];
			moonDistances = new double[julianDates.length];
			timeIntervals = new double[julianDates.length];
			siderealTimeIntervals = new double[julianDates.length];

		
			
			// for each julian date
			for (int j = 0; j < julianDates.length; j++) {
				timer.calcTimer(2);
				
				jDate = julianDates[j];				
				
				// calculate local sidereal time
				//lst =  getLST(jDate,observLongitude);
				
				lst = lsts[j] ;
				
				// calculate airmass
				airMass = getAirMassValue(jDate,lst,targetRA,targetDEC,observLatitude);
				//airMass = Convert.round(airMass, 6);								
				airMasses[j]= airMass;
				timer.calcTimer(2);			
								
				timer.calcTimer(3);	
				// calculate moon angular distance with target
				if ( calcMoonDist ) {				
					moonDist = getMoonDistValue(jDate,moonRAs[j],moonDecs[j],targetRA,targetDEC,observLongitude,observLatitude);
					moonDistances[j] = moonDist;
				}	
				
				Date utDate;
				timer.calcTimer(3);
				// calculate if the date falls into user defined time intervals
				
				timer.calcTimer(4);
				if ( calcTimeInterval) {
					utDate =  new Date(startTime.getTime() + (interval * j * 60000));
					timeInterval = VisibilityCalcUtils.getTimeIntervalValue(ob.getTimeIntervals(), utDate);				
					timeIntervals[j] = timeInterval;
				}
				timer.calcTimer(4);
				timer.calcTimer(5);
				
				// calculate if the date falls into user defined sidereal time intervals
				if ( calcSiderealTimeInterval ) {
					utDate =  new Date(startTime.getTime() + (interval * j * 60000));
					timeInterval = getSiderealTimeIntervalValue(ob.getSiderealTimeIntervals(), utDate, jDate);
					siderealTimeIntervals[j] = timeInterval;
				}
				timer.calcTimer(5);				
				
			}
			
			timer.calcTimer(6);
			
			// 	build the PCFs
			
			
			// moon pcfs

			double moonDistanceConstraint = ob.getObInfo().getMoonAngularDistance()
					- (ob.getObInfo().getMoonAngularDistance()* (moonDistanceTolerance / 100));
			double fliConstraint = ob.getObInfo().getFractionalLunarIllumination()
					+ (ob.getObInfo().getFractionalLunarIllumination()* (fliTolerance / 100));
			
			double [] moonDistanceBinary = getMoonDistanceBinary(moonDistances,moonDistanceConstraint,moonUps);

						
			double [] fliBinary = getMoonFLIBinary(flis,fliConstraint,moonUps);
				

			
			
			// airmass
			double airmassConstraint = ob.getObInfo().getAirmass()
					+ ob.getObInfo().getAirmass() * (airmassTolerance / 100);
	        double [] airmassBinary = getBinaryAirmass(airMasses,airmassConstraint);
	
	        
			
	        timer.calcTimer(6);
	        
	        timer.calcTimer(7);
	        
	        
	        // result pcf 
	        result = combineConstraints(startTime, interval, airMasses,
	        		moonDistanceBinary, sunAngles, timeIntervals,
					siderealTimeIntervals, fliBinary, moonUps,
					airmassConstraint,moonDistanceConstraint,fliConstraint, 
					options);	
	        
	        timer.calcTimer(7);
	        
	        timer.calcTimer(8);
	        
	        // apply zenith constraint
	        
	        if (calcZenithConstraint) {
	        	
	        	if ( entersZenithRegion(options.getZenithConstraint(),targetDEC, observLatitude) ) {
	        		double [] startEndZenith = findZenithAreaAnalitical(options.getZenithConstraint(),targetRA,targetDEC,observLatitude,observLongitude);
	        		result = confineZenithConstraint(lsts,startEndZenith[0],startEndZenith[1],result);
	        	}
	        }
	        
	        
	        timer.calcTimer(8);
            
            
	        timer.calcTimer(9);
	        
	        	    
	        
	        OBScheduleInfo info = new OBScheduleInfo();
	        execution = calcExecTimes(info,result,interval,startTime, ob.getObInfo().getExecutionTime());
	        
	        
			// add information to info						
			info.setOb(ob);								
			info.setAirmass(airMasses);
			info.setMeetsAirmassConstraint(airmassBinary);
			info.setMeetsZenithConstraint(meetZenithConstraint);
			info.setMoonAngle(moonDistances);
			info.setFli(flis);
			info.setMoonUp(moonUps);
			info.setSunDown(sunDowns);
			info.setResult(result);
			info.setTimeInterval(timeIntervals);
			info.setSiderealTimeInterval(siderealTimeIntervals);
			info.setExecTime(execution);
			
			info.setStart(startTime);
			info.setEnd(endTime);
			info.setInterval(interval);
			
			
			resInfo[i] = info;
			//System.out.println(info);
			timer.calcTimer(9);
			
			
			// updates the progress model
			setValue(getValue()+1);
			
		}
		
		
		
		stdlog_.debug("Visibility computation performance report:");
		stdlog_.debug("Cache hits: " + visibilityCache_.getCacheHits());
		stdlog_.debug("Cache no hits: " + visibilityCache_.getCacheNoHits());
		stdlog_.debug(timer.reportResult());
				
		return(resInfo);
		
		
	}




	/**
	 * Returns a binary array that considers if the moonDistanceConstraint is fulfilled
	 * 
	 * @param moonDistances
	 * @param moonDistanceConstraint
	 * @param moonUps
	 * @return
	 */
	private double[] getMoonDistanceBinary(double[] moonDistances, double moonDistanceConstraint, double[] moonUps) {
		double [] res = new double[moonDistances.length];
		
		for (int i = 0; i < moonUps.length; i++) {				
			double up = moonUps[i];	    		
			if ( up == 1.0 ) 	{
				if( moonDistances[i] <  moonDistanceConstraint ) 
					res[i] = 0;
				else 
					res[i] = 1;
				
			}
			else 
				res[i] = 1;
		}
		
		return(res);
	}
	
	
	/**
	 * Returns a binary array that considers if the moonDistanceConstraint is fulfilled
	 * 
	 * @param moonDistances
	 * @param moonDistanceConstraint
	 * @param moonUps
	 * @return
	 */
	private double[] getMoonFLIBinary(double[] flis, double fliConstraint, double[] moonUps) {
		double [] res = new double[flis.length];
		
		for (int i = 0; i < moonUps.length; i++) {				
			double up = moonUps[i];	    		
			if ( up == 1.0 ) 	{
				if( flis[i] <  fliConstraint ) 
					res[i] = 1;
				else 
					res[i] = 0;
				
			}
			else 
				res[i] = 1;
		}
		
		return(res);
	}

	

	protected double[] getSunDownBinary(double[] sunElevations, double sunConstraint) {
		double [] res = new double[sunElevations.length];
		
		for (int i = 0; i < sunElevations.length; i++) {				

			if( sunElevations[i] <  sunConstraint ) 
				res[i] = 1;
			else 
				res[i] = 0;
				
		}
		
		return(res);
	}
	
	
	


	/**
	 * Determines if a target enters the zenith avoidance region
	 * 
	 * @param zenithConstraint in degrees
	 * @param targetDEC in radians
	 * @param observLatitude in radians
	 * @return
	 */
	private boolean entersZenithRegion(double zenithConstraint, double targetDEC, double observLatitude) {
		return(Math.abs(targetDEC - observLatitude)<=Math.toRadians(zenithConstraint));
	}



	
	/**
	 * 
	 * @param zenithConstraint
	 * @param targetRA
	 * @param targetDEC
	 * @param observLatitude
	 * @param observLongitude
	 * @return a double[2] array 
	 */
	protected double [] findZenithAreaAnalitical(double zenithConstraint, double targetRA, double targetDEC,
			double observLatitude, double observLongitude) {
		
		double [] res = new double[2];
				
		double ha = Math.abs(Math.acos( ( Math.cos(Math.toRadians(zenithConstraint))
				  - (Math.sin(observLatitude) * Math.sin(targetDEC)))
				/ (Math.cos(observLatitude) * Math.cos(targetDEC))   ));
		
		double lst1 = targetRA - ha;
		double lst2 = targetRA + ha;				
		
		res[0] = lst1;
		res[1] = lst2;
		
		return(res);
	}
	
	
	

	/**
	 * Returns the airmass for an object in (ra,dec) position, with relation to observatory
	 * latitude and local sidereal time
	 * 
	 * @param ra Right ascension (radians)
	 * @param dec Declination (radians)
	 * @param obsLat observatory latitude (radians)
	 * @param lst Local sidereal time
	 * @return Airmass
	 */
	protected double airmass(double ra, double dec, double obsLat, double lst) {
		
		// zenith angle
		double za = zenithAngle(ra, dec, obsLat, lst);
		
		
		// airmass
		double airmass;
		
		/*
		 * Copied from SLALIB fortran source to calculate the airmass based on zenith angle
		 * 
		 * SECZM1 = 1D0/(COS(MIN(1.52D0,ABS(ZD))))-1D0 sla_AIRMAS = 1D0 +
		 * SECZM1*(0.9981833D0 : - SECZM1*(0.002875D0 + 0.0008083D0*SECZM1))
		 */
		
		double secz = 1 / (Math.cos(Math.min(1.52, Math.abs(za)))) - 1;
		airmass = 1 + secz * (0.9981833 - secz * (0.002875 + 0.0008083 * secz));
		
		return (airmass);
	}
	
	/**
	 * Calculates the zenith angle for an object in (ra,dec) position, 
	 * with relation to observatory latitude and local sidereal time
	 * 
	 * @param ra  
	 * @param dec  (radians)
	 * @param obsLat (radians)
	 * @param lst
	 * @return double (radians)
	 */
	protected double zenithAngle(double ra, double dec, double obsLat,
			double lst) {
		double el = VisibilityCalcUtils.getElevation(ra, dec, obsLat, lst);
 		// zenith angle
		double za = (Math.PI / 2) - el;
		return za;
	}


	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getAirMassValue(double, double, double, double, double)
	 */
	public double getAirMassValue(double jdate, double lst, double targetRA,
			double targetDEC, double lat) {
		
		Double airmass;
			
		if (cacheEnabled_) {
			
			//	Check the cache
			airmass = visibilityCache_.getAirmass(jdate,targetRA,targetDEC); 
			
			if (airmass == null) {
				airmass = new Double(airmass(targetRA, targetDEC, lat, lst));
				
				//store in cache
				visibilityCache_.putAirmass(jdate,targetRA,targetDEC,airmass.doubleValue());
			}
			
		} else {
			airmass = new Double(airmass(targetRA, targetDEC, lat, lst));
		}
		
		
		return airmass.doubleValue();
	}
	
	
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getFLIValue(double, double, double)
	 */
	public double getFLIValue(double jdate, double longit, double lat) {
		
		
		double d = 0;
		double fli = 0;
		
		Latitude latitude = new Latitude(Math.toDegrees(lat));
		Longitude longitude = new Longitude(Math.toDegrees(-longit));			
		
		try {		
			ObsInfo oi = new ObsInfo(latitude, longitude, 0);
			
			// obtain RA and DEC for the moon
			PlanetData pd = new PlanetData(Planets.LUNA, jdate, oi);
			
			double moonRA = AstroOps.normalizeRadians(pd.getRightAscension());
			double moonDec= pd.getDeclination();
			
			
			//	obtain RA and DEC for the sun
			pd = new PlanetData(Planets.SUN, jdate, oi);
			double sunRA = AstroOps.normalizeRadians(pd.getRightAscension());
			double sunDec= pd.getDeclination();
			
			
			// calculate angular distance between the two objects
			d = angularDistance(moonRA, moonDec, sunRA, sunDec);
			
			// calculate FLI based on this distance
			fli = (1.0 - Math.cos(d))/2;
			
		} catch (NoInitException e) {
			e.printStackTrace();
		}
		
		return fli;
	}
	
	
	/**
	 * Calculates the angular distance between two objects
	 * Took from Duffet-Smith book
	 * 
	 * @param ra1  RA of the first object (radians)
	 * @param dec1 DEC of the first object (radians)
	 * @param ra2 RA of the second object (radians)
	 * @param dec2 DEC of the secon object (radians)
	 * @return Angular distance (radians)
	 */
	public double angularDistance(double ra1, double dec1, double ra2, double dec2) {
		double d;
		
		// calculate the angular distance
		double cosD = Math.sin(dec1)*Math.sin(dec2) + Math.cos(dec1)*Math.cos(dec2)*Math.cos(ra1-ra2);
				
		double _10ArcMin = Math.toRadians(1./6.);
		
		double cos10ArcMin =  Math.cos(_10ArcMin);
		
		if ( cosD > cos10ArcMin ) {
			double deltaRA = ra2 - ra1;
			double deltaDEC = dec2 - dec1;
			
			double sqrCosDEC =  Math.cos(dec1)*Math.cos(dec1); 
			double sqrDeltaRA = deltaRA*deltaRA;
			double sqrDeltaDEC =  deltaDEC*deltaDEC;
			
			d = Math.sqrt(  sqrCosDEC*sqrDeltaRA + sqrDeltaDEC  );
		} else {
		
			d = Math.acos(cosD);
		}
		return d;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getLST(double, double)
	 */
	public double getLST(double jdate, double longitude) {
		
		Double lst;		
		
		if (cacheEnabled_) {
			
			lst = visibilityCache_.getLST(jdate);		
			
			if (lst == null ) {
				
				double gmst = AstroOps.greenwichSiderealTime(jdate);
				gmst = AstroOps.normalizeRadians(gmst);
				lst = new Double(gmst - longitude);		
				
				// store it in cache
				visibilityCache_.putLST(jdate,lst.doubleValue());
				
			} 
		} else {
			double gmst = AstroOps.greenwichSiderealTime(jdate);
			gmst = AstroOps.normalizeRadians(gmst);
			lst = new Double(gmst - longitude);
		}
		
		return (lst.doubleValue());
		
	}
	
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonDistValue(double, double, double, double, double)
	 */
	public double getMoonDistValue(double jdate, double moonRA , double moonDec, 
			double targetRA,
			double targetDEC, double longit, double lat) {
						
		
		// convert to radians
		moonRA =  Math.toRadians(moonRA);
		moonDec = Math.toRadians(moonDec);
		
		
		Double distance;
		
		if (cacheEnabled_) {
			
			//	Check the cache
			distance = visibilityCache_.getMoonDistance(jdate,targetRA,targetDEC); 
			
			if (distance == null) {
				distance = new Double(angularDistance(moonRA,moonDec,targetRA,targetDEC));
				
				//store in cache
				visibilityCache_.putMoonDistance(jdate,targetRA,targetDEC,distance.doubleValue());
			}				
		} else {
			distance = new Double(angularDistance(moonRA,moonDec,targetRA,targetDEC));
		}
		
		distance = Math.toDegrees(distance); 
		return distance;
		
		
	}
	
	
	
	
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getSunValue(double, double, double, double)
	 */
	public double getSunValue(double jdate, double longit, double lat,
			double lst) {
		
		Double sunAngle = null;
		
		
		if (cacheEnabled_) {
		
		//	Check the cache			
		sunAngle = visibilityCache_.getSunElevation(jdate);
		
		// check if it is cached
		if (sunAngle == null) {
			
			double tmp = sunElevation(jdate, longit, lat);
			
			sunAngle  = new Double(Math.toDegrees(tmp));
			
			
			// store it in cache
			visibilityCache_.putSunElevation(jdate,sunAngle.doubleValue());		
		}
		
		} else {
			double tmp = sunElevation(jdate, longit, lat);
			
			sunAngle  = new Double(Math.toDegrees(tmp));
		}
		
		return(sunAngle.doubleValue());
	}


	private double sunElevation(double jdate, double longit, double lat) {
		// calculate 
		double tmp = 0;
		Latitude latitude = new Latitude(Math.toDegrees(lat));
		Longitude longitude = new Longitude(Math.toDegrees(-longit));
		ObsInfo oi = new ObsInfo(latitude, longitude);
		PlanetData pd = new PlanetData(Planets.SUN, jdate, oi);		
		
		try {
			tmp = Math.asin(pd.getAltAzRadius()); //- risesetAlt;;
			
			//sunAngle = AstroOps.normalizeRadians(sunAngle);
		} catch (NoInitException e) {
			e.printStackTrace();
		}
		return tmp;
	}
	
	
	/*
	 * (non-Javadoc)	 
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonVisible(double,
	 *      double, double)
	 */
	public double getMoonVisible(double jdate, double longit, double lat) {
		
		double moonAngle = getMoonAngle(jdate, longit, lat);
		boolean visible = moonAngle >= 0.0;
		
		if (visible)
			return (1);
		else
			return (0);
	}
	
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonAngle(double, double, double)
	 */
	public double getMoonAngle(double jdate, double longit, double lat) {
		
		Double moonAngle = null;
		
		
		if (cacheEnabled_) {
			
			//	 Check the cache
			moonAngle = visibilityCache_.getMoonElevation(jdate);
			
			// check if it is cached
			if (moonAngle == null) {
				
				
				double tmp = moonElevation(jdate, longit, lat);
				moonAngle =  new Double(Math.toDegrees(tmp));
				
				// store it in cache
				visibilityCache_.putMoonElevation(jdate,moonAngle.doubleValue());
				
			}
			
		} else {
			double tmp = moonElevation(jdate, longit, lat);
			moonAngle =  new Double(Math.toDegrees(tmp));
		}
		
		return(moonAngle.doubleValue());
	}


	private double moonElevation(double jdate, double longit, double lat) {
		double tmp = 0;
		// calculate moon angle
		Latitude latitude = new Latitude(Math.toDegrees(lat));
		Longitude longitude = new Longitude(Math.toDegrees(-longit));
		ObsInfo oi = new ObsInfo(latitude, longitude, 0);
		PlanetData pd = new PlanetData(Planets.LUNA, jdate, oi);									
		try {
			tmp = Math.asin(pd.getAltAzRadius());			
		} catch (NoInitException e) {
			e.printStackTrace();
		}
		return tmp;
	}
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonAngle(double, double, double)
	 */
	public double getMoonRa(double jdate, double longit, double lat) {
		
		
		Double moonRa = null;
		
		
		if (cacheEnabled_) {
			// Check the cache
			moonRa = visibilityCache_.getMoonRa(jdate);
			
			if (moonRa == null) {
				
				
				double tmp = moonRA(jdate, longit, lat);
				
				moonRa = new Double(tmp);
				
				// store it in cache
				visibilityCache_.putMoonRa(jdate,moonRa.doubleValue());
			}
		} else {
			double tmp = moonRA(jdate, longit, lat);
			
			moonRa = new Double(tmp);
			
		}
		
		return moonRa.doubleValue();
	}


	private double moonRA(double jdate, double longit, double lat) {
		double tmp = 0;
		
		Latitude latitude = new Latitude(Math.toDegrees(lat));
		Longitude longitude = new Longitude(Math.toDegrees(-longit));
		
		ObsInfo oi = new ObsInfo(latitude, longitude, 0);
		PlanetData pd = new PlanetData(Planets.LUNA, jdate, oi);
		
		
		try {
			tmp = pd.getRightAscension();
		} catch (NoInitException e) {
			e.printStackTrace();
		}
		
		tmp = Math.toDegrees(tmp);
		tmp = AstroOps.normalizeDegrees(tmp);
		return tmp;
	}
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.phase2.visiplot.VisibilityCalc#getMoonAngle(double, double, double)
	 */
	public double getMoonDec(double jdate, double longit, double lat) {
		
		
		Double moonDec = null;
		
		
		if (cacheEnabled_) {
			// Check the cache
			moonDec = visibilityCache_.getMoonDec(jdate);
			
			if (moonDec == null ) {
				
				double tmp = moonDec(jdate, longit, lat);			
				moonDec = new Double(tmp);
				
				// store in cache
				visibilityCache_.putMoonDec(jdate,moonDec.doubleValue());
				
			}
		} else {
			double tmp = moonDec(jdate, longit, lat);			
			moonDec = new Double(tmp);
		}
		
		return(moonDec.doubleValue());
	}


	private double moonDec(double jdate, double longit, double lat) {
		double tmp = 0;
		Latitude latitude = new Latitude(Math.toDegrees(lat));
		Longitude longitude = new Longitude(Math.toDegrees(-longit));
		
		ObsInfo oi = new ObsInfo(latitude, longitude, 0);
		PlanetData pd = new PlanetData(Planets.LUNA, jdate, oi);
		
		try {
			tmp = pd.getDeclination();
		} catch (NoInitException e) {
			e.printStackTrace();
		}
		tmp = Math.toDegrees(tmp);
		return tmp;
	}
	

	
	
	 /**
     * getTimeIntervalValue - get the Sidereal Time Interval Value If the time
     * interval has not been defined in the OB any time period is satisfactory
     * set the returned value to 1. Else if the start and end time does not fall
     * within this interval time (precision) set returned value to 0.
     * @param ObservationBlock
     * @param julian Date double
     * @return time interval value
     */
    public double getSiderealTimeIntervalValue(
    		TimeInterval[] timeInts,
        Date utDate,
        double jdate) {

        // Convet the jdate to LST in radioans.

        //Convert user values into radian
        //Start and end dates for ST Times.
        // If the end date is < start date then 
        // add 2 PI to end date. 

        // Then compate the values         
        double timeValue = 1.;
      
        
        double siderealTimeInRadians = this.getLST(jdate,VisibilityCalcUtils.PLONG);
        if (siderealTimeInRadians < 0) {
            siderealTimeInRadians = siderealTimeInRadians + (2 * Math.PI);
        }
        if (siderealTimeInRadians > (2 * Math.PI)) {
            siderealTimeInRadians = siderealTimeInRadians - (2 * Math.PI);
        }
        if (timeInts != null && timeInts.length != 0) {
            for (int i = 0; i < timeInts.length; i++) {
                double startTime =
                    timeInts[i].getStartTime() * (Math.PI / (12 * 3600));
                double endTime = timeInts[i].getEndTime() * (Math.PI / (12 * 3600));
                if (((siderealTimeInRadians >= startTime)
                    && (siderealTimeInRadians <= endTime))
                       || ((siderealTimeInRadians + (2 * Math.PI) >= startTime)
                        && (siderealTimeInRadians + (2 * Math.PI) <= endTime))) {
                    timeValue = 1.;
                    return (timeValue);
                } else {
                    timeValue = 0.;

                }
            }
        }
        return (timeValue);
    }
    
    /**
     * Iterates through the airmass array, 
     * and determines the intervals within which an OB is observable.
     * Adds these results to schedInfo object, 
     * and cuts the non-observable periods from the result
     * It returns an array with the execution time periods
     * 
     * @param schedInfo
     * @param resultPCF
     * @param execTime of the OB in seconds
     * @return 
     */
    protected double [] calcExecTimes(OBScheduleInfo schedInfo , double [] result, int interval , Date startDate,  int execTime) {  	
        int msecsInterval = VisibilityCalcUtils.MILLISECS_PER_SEC * 60 * interval;    	                       
        int length = result.length;                
        int execTimeMilli = execTime* VisibilityCalcUtils.MILLISECS_PER_SEC;                
        boolean inside = false; // whether if you are inside a visible interval or not
        
        double [] execTimeRes = new double[length]; 
        
        Date start = null;
        Date end = null;
        
        Date currDate = new Date(startDate.getTime());
        
        int i = 0;
        //find intervals 
        for (i = 0; i < length; i++) {
            double value = result[i];            
            execTimeRes[i] =  0;
            
            // found a visible point
            if (value >= 0  && value < AirMass.NOT_VIS) {
            	if (!inside ) {
            		start = currDate;            		
            		inside = true;            		
            	}

            // not visible 
            } else {            	             	
            	// found the end of a visible interval
            	if ( inside ) {
            		inside = false;	     	
            		end = currDate;
            		
            		markNotVisible(interval, i, execTime, execTimeRes) ;
            		
            		
            		// check if the execution time fits the interval
            		long intervalSize = end.getTime() - start.getTime();            		            		
            		if ( intervalSize >= execTimeMilli ) {
            			
            			// it fits, lets define when can the OB start being executed within the interval            			
            			end = new Date( end.getTime() - execTimeMilli  );
            			schedInfo.addObservableInterval(start,end);            			
            			            			
            			
            			
            		}
            		
            		

            	}		            	
            }

            currDate = new Date(currDate.getTime() + msecsInterval  );
        }
        
        // end of the loop, if its inside a visible interval, insert it 
        if ( inside ) {
        	end = currDate;
        	
        	markNotVisible(interval, i, execTime, execTimeRes) ;
        	
        	// 	 check if the execution time fits the interval
    		long intervalSize = end.getTime() - start.getTime();            		            		
    		if ( intervalSize >= execTimeMilli ) {
    			//  it fits, lets define when can the OB start being executed within the interval            			
    			end = new Date( end.getTime() - execTimeMilli );
    			schedInfo.addObservableInterval(start,end);
    			
    			
    			
    		}
        }
        
        
        return execTimeRes;
        
    }
    

    /**
     * Changes execution array to mark a part of it not visible, given a step interval,
     * execution time needed and an index to start from
     * It marks not visibile from that index back  
     * 
     * @param interval in minutes
     * @param index 
     * @param execTime in seconds
     * @param execTimeRes array to be 
     */
    private void markNotVisible(int interval, int index, int execTime, double [] execTimeRes) {
 
    	int execTimeMins = execTime/60;
    	
		int grayAreaIndex = (index -  execTimeMins/interval );
		if (grayAreaIndex >= 0) {
			for (int j = grayAreaIndex; j <= index; j++) {
				if ( j < execTimeRes.length )
					execTimeRes[j] = 1;
			}
		}
    }
    
    /**
	 * Combine all of the selected PCFs
	 * @return result pcf
	 */
	private double [] combineConstraints( Date start, int interval,
			double [] airmass,
			double [] moonDistBinary,
			double [] sun,
			double [] timeInterval,
			double [] siderealTimeInterval,
			double [] fliBinary,
			double [] moonUp,
			double airmassOBConstraint,
			double moonOBDistConstraint,
			double fliOBConstraint,
			VisicalcOptions options
		) {
		
		// initialize airmass array
	    double [] res = new double[airmass.length];
	    for (int i = 0; i < res.length; i++) {
	    	res[i] = 1;
		}
	
	    // if the moon is taken into account
	    if (options.calcMoon()) {	        	     	    
	    	
	    	
	    	// when the moon is down the value should be untouched, otherwise is the
	    	// conjunction of both FLI and moonDistance	    	
	    	for (int i = 0; i < moonUp.length; i++) {				
	    		double up = moonUp[i];	    		
	    		if ( up == 1.0 ) {	    		
	    			if (options.calcFli()) {
	    				res[i] = res[i] * fliBinary[i];
	    			}		        
	    			if (options.calcMoonDist()) {
	    				
	    				res[i] = res[i] * moonDistBinary[i];
	    			}
	    		}
	    	}
	    }
	    
	    //creates the sun binary array
	    double [] sunDownBinary = new double[sun.length];
	    for (int i = 0; i < sunDownBinary.length; i++) {
			
	    	if ( sun[i] < options.getSunConstraint())
	    		sunDownBinary[i] = 1;
	    	else
	    		sunDownBinary[i] = 0;
		}
	    
	    if (options.calcSun()) {
        	res = VisibilityCalcUtils.conjunction(res,sunDownBinary);
        }
	    
	    if (options.calcTimeInterval()) {
	    	res = VisibilityCalcUtils.conjunction(res,timeInterval);
	    }
	    if (options.calcSiderealTimeInterval()) {
	    	res = VisibilityCalcUtils.conjunction(res,siderealTimeInterval);
	    }
	    
	    	    	    
	    double[] binaryAirmass = getBinaryAirmass(airmass, airmassOBConstraint);	    
	    res = VisibilityCalcUtils.conjunction(res,binaryAirmass);
	    
	    // iterate over the binary result, and replace 1 for the correct airmass
	    // and replace 0 for AirMass.NOT_VIS
	    for (int i = 0; i < res.length; i++) {
	    	
	    	if (res[i] == 1) {	    		
	    		if (airmass[i] < AirMass.NOT_VIS ) {
	    			res[i] = airmass[i];	    	
	    		}
	    	}	else {
	    		res[i] = AirMass.NOT_VIS;
	    	}
		}		    	      
	    

	    return (res);
	}


	/**
	 * Gets a binary array of the airmass (1 where the constraint is fulfilled, and 0 where not)
	 * 
	 * @param airmass
	 * @param airmassOBConstraint
	 * @return
	 */
	private double[] getBinaryAirmass(double[] airmass, double airmassOBConstraint) {	    
	    double [] binaryAirmass = new double[airmass.length];
	    for (int i = 0; i < airmass.length; i++) {	    	
	    	double airm = airmass[i];
	    	if ( airm < AirMass.NOT_VIS  && airm < airmassOBConstraint )
	    		binaryAirmass[i] = 1;
	    	else
	    		binaryAirmass[i] = 0;
		}
	    	    
	    
	    double [] bin = binaryAirmass;
		return bin;
	}



	
	
	/**
	 * confine with the meet zenith constraint array
	 * the zenith constraint array has bigger precision (smaller step) than
	 * the airmass array
	 * 
	 * @param interval
	 * @param meetsZenithConstraint
	 * @param options
	 * @param res
	 */
	private double[] confineZenithConstraint(double[] lsts, double startLST, double endLST, double[] res) {

		double[] result = res;

		
		for (int i = 0; i < result.length; i++) {

			if (startLST <= lsts[i] && lsts[i] <= endLST)
				result[i] = AirMass.NOT_VIS;

		}

		return result;
	}
	
	
	
	/**
	 * 
	 * Dummy implementation of a sun rise time algorithm It simple iterates
	 * (using a stepInterval) until it finds a sun angle bigger than riseAngle
	 * 
	 * @param riseAngle
	 *            in degrees
	 * @param stepInterval
	 *            in minutes
	 * @param start
	 *            when to start calculating
	 * @param longit
	 *            longitude of observatory in radians
	 * @param lat
	 *            latitude of observatory in radians
	 * @param lst
	 *            local sidereal time of observatory
	 * @return
	 */
	public Date getSunRiseTime( 
			double riseAngle , 
			int stepInterval,  
			Date start ,  
			double longit, 
			double lat) {
			
					
		Date d = new Date(start.getTime());
								
		
		double jdate = JulianDate.getJDC(d);
		double lst = getLST(jdate,longit);
		double sunAngle = getSunValue(jdate,longit,lat,lst);
						
		boolean riseNext = true;
		if ( sunAngle > riseAngle )
			riseNext = false;
		
		

		while ( sunAngle < riseAngle || riseNext == false ) {
			d = new Date(d.getTime() + VisibilityCalcUtils.MILLISECS_PER_SEC*60*stepInterval  );  
			jdate = JulianDate.getJDC(d);				
			sunAngle = getSunValue(jdate,longit,lat,0);			

			if (sunAngle < riseAngle ) 
				riseNext  = true;
		}
			
		
		return(d);		
	}
	
	
	
	
    // ProgressModel interface implementation
    
	public boolean isCacheEnabled() {
		return cacheEnabled_;
	}


	public void setCacheEnabled(boolean cacheEnabled_) {
		this.cacheEnabled_ = cacheEnabled_;
	}


	public int getMaximum() {
		return(progressModel.getMaximum());
	}

	public int getMinimum() {
		return(progressModel.getMinimum());
	}

	public int getValue() {
		return(progressModel.getValue());
	}

	public void setMaximum(int max) {
		progressModel.setMaximum(max);
	}

	public void setMinimum(int min) {
		progressModel.setMinimum(min);
	}

	public void setValue(int value) {
		progressModel.setValue(value);	
	}

	public String getMsg() {
		return(progressModel.getMsg());
	}

	public void setMsg(String msg) {
		progressModel.setMsg(msg);		
	}

	public void addChangeListener(ProgressModelListener l) {
		progressModel.addChangeListener(l);
		
	}

	public void removeChangeListener(ProgressModelListener l) {
		progressModel.removeChangeListener(l);
		
	}


}
