package org.eso.ohs.phase2.visibility;

import java.util.Date;

import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/*
 * This class contains the constraints and options for visibility calculation of an OB
 * 
 * Created on Aug 19, 2005
 * 
 * @author pnunes 
 *
 */
public class VisicalcOptions {
	
	/**
     * Config keyword for telescope zenith constraint
     */
    public static final String telescopeConstraintKeyword = "TELESCOPE.ELEVATION.CONSTRAINT";
    
    
    /*
     * Config keyword for delay on the start time of the computation
     */    
    public static final String startTimeDelayKeyword = "ORANG.STARTTIME.DELAY";
    public static final  int DEFAULT_START_TIME_DELAY = 0;
    
    
	private double moonDistanceTolerance; // tolerance of moon distance to target in percentage
	private double airmassTolerance; // airmass tolerance in percentage
	private double fliTolerance; // fli tolerance in percentage
	private double sunConstraint;  // sun elevation angle in degrees
	private double moonConstraint;  // moon elevation angle in degrees
	private double zenithConstraint;  // maximum elevation in degrees	
	
	private boolean calcAirmass;
	private boolean calcFli;
	private boolean calcMoonDist;
	private boolean calcMoon; 
	private boolean calcSun;
	private boolean calcTimeInterval; 
	private boolean calcSiderealTimeInterval;
	private boolean calcZenithInvisibility;
	private boolean calcExecTime;
	
	private Date startTime;
	private Date endTime;
	
	private int interval = 10;

	private int startTimeDelay = 0;


    private boolean executableCheck;;

	
	
	



	/**
	 * Default constructor 
	 */
	public VisicalcOptions() {
		
		initZenithConfig();
		initStartTimeDelayConfig();
	   
		
	}



	private void initZenithConfig() {
		if (AppConfig.getAppConfig() != null) {
			zenithConstraint = 
				new Double(AppConfig.getAppConfig().getValue(
						VisicalcOptions.telescopeConstraintKeyword)).doubleValue();			
		}
	}
	
	private void initStartTimeDelayConfig() {
		if (AppConfig.getAppConfig() != null) {
			startTimeDelay =AppConfig.getAppConfig().getInteger(
					VisicalcOptions.startTimeDelayKeyword,DEFAULT_START_TIME_DELAY);
		}
	}
	
	
	
	public double getAirmassTolerance() {
		return airmassTolerance;
	}
	public void setAirmassTolerance(double airmassTolerance) {
		this.airmassTolerance = airmassTolerance;
	}
	public boolean calcFli() {
		return calcFli;
	}
	public void setCalcFli(boolean calcFli) {
		this.calcFli = calcFli;
	}
	public boolean calcMoon() {
		return calcMoon;
	}
	public void setCalcMoon(boolean calcMoon) {
		this.calcMoon = calcMoon;
	}
	public boolean calcMoonDist() {
		return calcMoonDist;
	}
	public void setCalcMoonDist(boolean calcMoonDist) {
		this.calcMoonDist = calcMoonDist;
	}




	public boolean calcSiderealTimeInterval() {
		return calcSiderealTimeInterval;
	}
	public void setCalcSiderealTimeInterval(boolean calcSiderealTimeInterval) {
		this.calcSiderealTimeInterval = calcSiderealTimeInterval;
	}
	public boolean calcSun() {
		return calcSun;
	}
	public void setCalcSun(boolean calcSun) {
		this.calcSun = calcSun;
	}
	public boolean calcTimeInterval() {
		return calcTimeInterval;
	}
	public void setCalcTimeInterval(boolean calcTimeInterval) {
		this.calcTimeInterval = calcTimeInterval;
	}
	public double getFliTolerance() {
		return fliTolerance;
	}
	public void setFliTolerance(double fliTolerance) {
		this.fliTolerance = fliTolerance;
	}
	public double getMoonConstraint() {
		return moonConstraint;
	}
	public void setMoonConstraint(double moonConstraint) {
		this.moonConstraint = moonConstraint;
	}
	public double getMoonDistanceTolerance() {
		return moonDistanceTolerance;
	}
	public void setMoonDistanceTolerance(double moonDistanceTolerance) {
		this.moonDistanceTolerance = moonDistanceTolerance;
	}
	public double getSunConstraint() {
		return sunConstraint;
	}
	public void setSunConstraint(double sunConstraint) {
		this.sunConstraint = sunConstraint;
	}
	
	
	
	
	public boolean calcZenithInvisibility() {
		return calcZenithInvisibility;
	}

	public void setCalcZenithInvisibility(boolean calcZenithInvisibility) {
		this.calcZenithInvisibility = calcZenithInvisibility;
	}

	
	
	
	public double getZenithConstraint() {
		return zenithConstraint;
	}

	public void setZenithConstraint(double zenithConstraint) {
		this.zenithConstraint = zenithConstraint;
	}

	/**
	 * getSunConstraint calls isReadFromOB() to determines if the default value
	 * of -18 or the VisiPlot Options value is used
	 * 
	 * @return Sun Constraint
	 * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
	 * @see VisiPlotOptions#isReadFromOB(String value)
	 */
	private double getSunConstraint(VisiPlotOptions vpo, String value) {
		boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
		double sunConstraint;

		if (!readFromOB) {
			try {
				sunConstraint = Double.valueOf(value).doubleValue();
			} catch (NumberFormatException e) {
				sunConstraint = -18.;
			}
		} else {
			sunConstraint = -18.;
		}
		// System.out.print(" visiplot.Visiplot.getSunConstraint " +
		// sunConstraint_);
		return (sunConstraint);
	}
    
    
    /**
	 * getMoonDistance calls isReadFromOB() to determines which constraint value
	 * to use; the VisiPlot Options value or the constraint from the database.
	 * 
	 * @return Moon Distance Constraint
	 * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
	 * @see VisiPlotOptions#isReadFromOB(String value)
	 */
	private double getMoonConstraint(VisiPlotOptions vpo,
			String value) {

		boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
		double moonConstraint;

		if (!readFromOB) {
			try {
				moonConstraint = Double.valueOf(value).doubleValue();
			} catch (NumberFormatException e) {
				moonConstraint = 0.;
			}
		} else {
			moonConstraint = 0.;
		}
		//stdlog_.info(" visiplot.Visiplot.getMoonConstraint " + moonConstraint_ + " " + value);
		return (moonConstraint);
	}



	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public int getInterval() {
		return interval;
	}

	public void setInterval(int interval) {
		this.interval = interval;
	}



	public boolean calcAirmass() {
		return calcAirmass;
	}


	public void setCalcAirmass(boolean calcAirmass) {
		this.calcAirmass = calcAirmass;
	}



	public boolean calcExecTime() {
		return calcExecTime;
	}



	public void setCalcExecTime(boolean calcExecTime) {
		this.calcExecTime = calcExecTime;
	}



	public int getStartTimeDelay() {
		return startTimeDelay;
	}



    public boolean getExecutableCheck() {
      
        return executableCheck;
    }



    public void setExecutableCheck(boolean executableCheck) {
        this.executableCheck = executableCheck;
        
    }
}
