package org.eso.ohs.phase2.visiplot;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.TimeZone;
import java.util.Vector;

import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.orang.ObDataContainer;
import org.eso.ohs.phase2.orang.OrangCache;
import org.eso.ohs.phase2.orang.OrangCacheContainer;
import org.eso.ohs.phase2.visibility.PCF;
import org.eso.ohs.phase2.visibility.PCFInterval;
import org.eso.ohs.phase2.visibility.VisibilityJCalc;
import org.eso.ohs.phase2.visibility.VisicalcOptions;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter.ValueRejectedException;

/**
 * This class contains methods used by visiplots to prepare the data for plotting
 *
 * @author pnunes 20.10.2005
 *
 */
public class VisiplotUtils {

	
    private static final String HOUR_FORMAT = "HH.mm'h'";
    // Text report values.
    private static String SPACE = "      ";    
    private static String UNKNOWN = "unknown";
	
    private final static SimpleDateFormat formatter = new SimpleDateFormat(HOUR_FORMAT);
    
  
    static {
        formatter.setTimeZone(TimeZone.getTimeZone("GMT"));
    }
    
    
	/**
	 * Constructor
	 */
	public VisiplotUtils() {
		
	}
	
	
	 /**
     * Group similar Obs.
     * When all the inputs to the OB PCF functions for 2 OBS are identical,
     * the resulting PCF will be identical, and therefore the OBS can
     * be grouped together under a single visibility plot. If the user
     * specifies it (second parameter is true).
     * This method accepts an array of OBs as input and returns an array
     * of OB arrays. Each OB array contains groups of identical OBs.
     *
     * @param observation block array to be grouped
     * @param group identical obs. If false, an ObservationBlock[][] object
     * is returned containing a set of ObservationBlock[] array, each with
     * a single ObservationBlock element.
     * @return array of OB arrays, grouped if the group param is true.
     */

    public static ObDataContainer[][] groupDuplicates(
        ObDataContainer[] in,
        boolean groupObs) {
    	ObDataContainer rc[][] = null;
        if (groupObs) {
            Hashtable<String, Vector<ObDataContainer>> groups = new Hashtable<String, Vector<ObDataContainer>>();
            for (int i = 0; i < in.length; i++) {
                // Integer hash = getHashCode(in[i]);
                String hash = getHashCode(in[i]);
                Vector<ObDataContainer> group = groups.get(hash);
                if (group == null) {
                    group = new Vector<ObDataContainer>();
                    groups.put(hash, group);
                }
                group.addElement(in[i]);
            }

            // copy from the Hashtable of Vectors to a OB[][] table.
            rc = new ObDataContainer[groups.size()][];
            int x = 0;
            for (Enumeration<Vector<ObDataContainer>> grp = groups.elements();
                grp.hasMoreElements();
                x++) {
                Vector group = grp.nextElement();
                ObDataContainer[] blks = new ObDataContainer[group.size()];
                rc[x] = blks;
                int i = 0;
                for (Enumeration elem = group.elements();
                    elem.hasMoreElements();
                    i++) {
                    blks[i] = (ObDataContainer) elem.nextElement();
                }
            }
        } else {
            // do not group OBs.
            rc = new ObDataContainer[in.length][];
            for (int i = 0; i < in.length; i++) {
                rc[i] = new ObDataContainer[1];
                rc[i][0] = in[i];
            }
        }
        return rc;
    }
    
    
    /**
     * Generate the hashcode data used for OB comparison.
     * Hashcode is generated from the following data (all report inputs,
     * besides OB ID and Program ID):
     *     - ra, dec
     *     - exec time
     *     - seeing, FLI, moon angle, sky trans
     *     - rank class
     * There is nothing fancy about this hascode - it is simply appending
     * all the report string together and returning it.
     * @return String hashcode for the OB.
     */

    private static String getHashCode(ObDataContainer ob) {
        StringBuffer data = new StringBuffer();        
        
        data.append("" + ob.getObInfo().getRa() + "\n");
        data.append("" + ob.getObInfo().getDec() + "\n");

        /* Noot needed anymore - pnunes 03.11.2005
         * 
        String execTime = null;
        try {
            if (EvmExecutorImpl.evmConfigured()) {
                EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                //Check that the evm is initialized
                if (evm.isInitialized()) {
                    evm.calculateETCS(ob);
                }
            }
        }         
        catch(EVMException e) {
            execTime = UNKNOWN;
        }
        catch (Exception e) {
            // there are occational errors when calculating exec time.
            // these can be caught here without interrupting the rest
            // of the hashcode generation.
            execTime = UNKNOWN;
        }
        
        
        if(execTime == null) {        
            execTime = Convert.secToHHMMSS((int) ob.getExecutionTime());
        }
        */
        
        String execTime = Convert.secToHHMMSS(ob.getObInfo().getExecutionTime());
        data.append("" + execTime + "\n");
        data.append("" + ob.getSchedRun().getRankClass() + "\n");

        
        data.append("" + ob.getObInfo().getSeeing() + "\n");
        data.append("" + ob.getObInfo().getFractionalLunarIllumination() + "\n");
        data.append("" + ob.getObInfo().getMoonAngularDistance() + "\n");
        data.append("" + ob.getObInfo().getSkyTransparency() + "\n");
        data.append("" + ob.getObInfo().getAirmass() + "\n");
        TimeInterval [] tis = ob.getTimeIntervals();
        if(tis != null && tis.length > 0) {
            for(int i=0;i<tis.length;i++) {
                data.append(tis[i].toReportString());
            }            
        }
        data.append("\n");
        tis = ob.getSiderealTimeIntervals();        
        if(tis != null && tis.length > 0) {
            for(int i=0;i<tis.length;i++) {
                data.append(tis[i].toReportStringSidereal());
            }
        }
        data.append("\n");                        
        return data.toString();
    }
    
    
    /**
     * Generate a text report for each group of OBs. The method assumes each
     * ObservationBlock[] element has at least one OB.
     * @return the chart text report from the ob according to the options.
     * The report is an array of String obects, each object is a line of text.
     */

    public static String[] getTextReport(
    		ObDataContainer[] obs,
        VisiPlotOptions opts) {
        String rc[] = new String[2];
       
        StringBuffer line1 = new StringBuffer();
        StringBuffer line2 = new StringBuffer();

        // for most of the report we only require the first OB in the array
        // as the OBs are grouped with any identical values.
        ObDataContainer ob = obs[0];
        
        OrangCache obCache = OrangCacheContainer.getOrangCache();
        ObDataContainer obOrang = null;
        if (obCache != null) {
         obOrang = obCache.getObFromId(ob);
        }
        // Text report line 1.
        if (opts.reportShowProgId()) {
            // report prog ids for all OBs passed.
            StringBuffer progs = new StringBuffer();
            for (int i = 0; i < obs.length; i++) {
            	ObDataContainer oblk = obs[i];
                String prog = new String(oblk.getSchedRun().getProgId());
                if (i > 0) {
                    progs.append(", ");
                }
                progs.append(prog);
            }
            line1.append(progs.toString() + SPACE);
        }

        if (opts.reportShowRA()) {
            
            line1.append("RA: ");
            if (ob.getObInfo()!= null) {
                String ra = Convert.millisecToHHMMSS(ob.getObInfo().getRa());
                line1.append(ra);
            } else {
                line1.append(UNKNOWN);
            }
            line1.append(SPACE);
        }

        if (opts.reportShowDec()) {
            
            line1.append("Dec: ");
            if (ob.getObInfo() != null) {
                String dec =
                    Convert.milliarcsecToDDMMSS(ob.getObInfo().getDec());
                line1.append("" + dec);
            } else {
                line1.append(UNKNOWN);
            }
            line1.append(SPACE);
        }

        // add OB count
        line1.append("OB Count: " + obs.length + SPACE);

        if (opts.reportShowExecTime()) {
        	
        
            String execTime = Convert.secToHHMMSS(ob.getObInfo().getExecutionTime());
            
            line1.append("Exec time: " + execTime + SPACE);
            
            
            
        }

        if (opts.reportShowOBId()) {
            // report ids for all OBs passed.
            StringBuffer obids = new StringBuffer("OBs: ");
            for (int i = 0; i < obs.length; i++) {
            	ObDataContainer oblk = obs[i];
                String obid ="" + oblk.getObInfo().getDbaseId();
                
       
                if (i > 0) {
                    obids.append(", ");
                }
                obids.append(obid);
            }
            line1.append(obids.toString() + SPACE);
        }

        // Text report line 2.

        if (opts.reportShowSeeing()) {
        	
            String val = (ob.getObInfo() != null) ? "" + ob.getObInfo().getSeeing() : UNKNOWN;
            line2.append("Seeing: " + val + SPACE);
        }

        if (opts.reportShowAirMass()) {
            String val = (ob.getObInfo() != null) ? "" + ob.getObInfo().getAirmass() : UNKNOWN;
            line2.append("Air Mass: " + val + SPACE);
        }

        if (opts.reportShowFLI()) {
            String val =
                (ob.getObInfo() != null)
                    ? "" + ob.getObInfo().getFractionalLunarIllumination()
                    : UNKNOWN;
            line2.append("FLI: " + val + SPACE);
        }

        if (opts.reportShowMoonDist()) {
            String val =
                (ob.getObInfo() != null) ? "" + ob.getObInfo().getMoonAngularDistance() : UNKNOWN;
            line2.append("Moon-dist: " + val + SPACE);
        }

        if (opts.reportShowSkyTrans()) {
            String st = (ob.getObInfo() != null) ? ob.getObInfo().getSkyTransparency() : null;
            if (st == null) {
                st = "---";
            }
            line2.append("Sky: " + st + SPACE);
        }

        if (opts.reportShowRank()) {
            String rankClass = ob.getSchedRun().getRankClass();
            if (rankClass == null || rankClass.trim().length() == 0) {
                rankClass = UNKNOWN;
            }
            line2.append("RankClass: " + rankClass + SPACE);
        }

        if (opts.reportShowStartTime()) {
            line2.append(
                "Start Time: "
                    + opts.getStartTimeISO()
                    + SPACE);
        }
    
        if (opts.reportShowWind() && obOrang != null && obOrang.getWindDistance() >= 0) {
            line2.append(
                "Wind Angle: "
                    + Math.round(obOrang.getWindDistance())
                    + SPACE);
        }

        // remove extra spaces at beginning and end of strings.
        rc[0] = line1.toString().trim();
        rc[1] = line2.toString().trim();

        return rc;
    }
    
    
    /**
     * Extract the appropriate sort key from the ob according to
     * the key named secified. All key names are taken from the
     * VisiPlotOptions class.
     * @param ObservationBlock containing key data
     * @param String name of key to be extracted from the OB
     * @return the key value, or "" if not found.
     */

    private static String getKey(ObDataContainer blk, String key) {
        String rc = null;
        if (key.equals(VisiplotOptionsHandler.sortByProgid)) {
            rc = blk.getSchedRun().getProgId();
        } else if (key.equals(VisiplotOptionsHandler.sortByObid)) {
            Media device =
                ObjectManager.getObjectManager().lookup(blk.getObInfo().getId()).getMedia();
            if (device.equals(Media.DBASE)) {
                rc = "" + blk.getObInfo().getDbaseId();
            } else {
                if (blk.getObInfo().getDbaseId() != 0) {
                    rc = "" + Config.getCfg().uniqueToTableId(blk.getObInfo().getDbaseId());
                } else {
                    rc = "" + blk.getObInfo().getId();
                }
            }
        } else if (key.equals(VisiplotOptionsHandler.sortByRank)) {
            rc = blk.getSchedRun().getRankClass();
        } else if (key.equals(VisiplotOptionsHandler.sortBySeeing)) {            
            rc = "" + blk.getObInfo().getSeeing();
        } else if (key.equals(VisiplotOptionsHandler.sortByFli)) {
            rc = "" + blk.getObInfo().getFractionalLunarIllumination();
        } else if (key.equals(VisiplotOptionsHandler.sortByMoondist)) {
            rc = "" + blk.getObInfo().getMoonAngularDistance();
        } else if (key.equals(VisiplotOptionsHandler.sortBySkytrans)) {
            rc = blk.getObInfo().getSkyTransparency();
        }
        if (rc == null) {
            rc = "";
        }
        return rc;
    }

    
    
    /**
     * Sort the observation blocks according to the setting options.
     * Each element to be sorted is an ObservationBlock[] containing
     * sets of identical OBs. The sorting is based on the values in the
     * first OB element in the array.
     * The method assumes there is an element in index 0 of the array.
     */

    public static ObDataContainer[][] sort(
    		ObDataContainer[][] obs,
        VisiPlotOptions opts) {

    	ObDataContainer[][] rc;
        // get the users choice of sort key.
        String keyName = opts.getSortPlot();
        if (keyName == null) {
            // no sort criteria found.
            rc = obs;
        } else if (
            keyName.equals(VisiplotOptionsHandler.sortByObid)
                || keyName.equals(VisiplotOptionsHandler.sortBySeeing)
                || keyName.equals(VisiplotOptionsHandler.sortByFli)
                || keyName.equals(VisiplotOptionsHandler.sortByMoondist)) {
            // NUMERIC based sort here
            // sort the OB according to the users specification

            SortableNumberElem toBeSorted[] =
                new SortableNumberElem[obs.length];
            for (int i = 0; i < obs.length; i++) {
                toBeSorted[i] =
                    new SortableNumberElem(
                        obs[i],
                        getKey(obs[i][0], keyName));
            }
            TextUtils.SortableNumber sorted[] = TextUtils.sort(toBeSorted);
            ObDataContainer sortedObs[][] = new ObDataContainer[obs.length][];
            for (int i = 0; i < toBeSorted.length; i++) {
                sortedObs[i] =
                    ((SortableNumberElem) sorted[i]).getOBs();
            }
            rc = sortedObs;

        } else {
            // STRING based sort here
            // sort the OB according to the users specification

            SortableElem toBeSorted[] =
                new SortableElem[obs.length];
            for (int i = 0; i < obs.length; i++) {
                toBeSorted[i] =
                    new SortableElem(
                        obs[i],
                        getKey(obs[i][0], keyName));
            }
            TextUtils.Sortable sorted[] = TextUtils.sort(toBeSorted);
            ObDataContainer sortedObs[][] = new ObDataContainer[obs.length][];
            for (int i = 0; i < toBeSorted.length; i++) {
                sortedObs[i] = ((SortableElem) sorted[i]).getOBs();
            }
            rc = sortedObs;
        }
        return rc;
    }
    
    
    /**
     * Finds an OB in an array, given its ID
     * 
     * @param obId
     * @param obs
     * @param visiplotOptions_ 
     * @return
     */
    public static OBScheduleInfo find( long obId , OBScheduleInfo [] obs, VisiPlotOptions visiplotOptions_ ) {
    	OBScheduleInfo res = null;
    	long paramObid = Config.getCfg().uniqueToTableId(obId);
    	
    	for (int i = 0; i < obs.length; i++) {
			OBScheduleInfo currentOb = obs[i];			
			long currentOBid = currentOb.getID();        
			if ( currentOBid == paramObid ) {
                // build again the obschedule info if it doesn't have the proper pcf
                if (currentOb.getResultPCF() == null) {           
                    VisicalcOptions visicalcOptions = visiplotOptions_.getVisicalcOptions();
                    VisibilityJCalc visibilityCalc = new VisibilityJCalc();
                    int interval = visiplotOptions_.getInterval();
                    Date endTime = visiplotOptions_.getEndTime();
                    Date startTime = visiplotOptions_.getStartTime();    
                    ObDataContainer[] obContainers = new ObDataContainer[1];
                    obContainers[0]= currentOb.getOb();
                    OBScheduleInfo[] newObs = null;
                    try {
                       newObs = visibilityCalc.calc(obContainers,interval,startTime,endTime,visicalcOptions,AppConfig.getAppConfig().getLatitude(),AppConfig.getAppConfig().getLongitude());
                       currentOb = newObs[0];
                    } catch (ValueRejectedException e) {               
                     MsgManager.errmsg("Cannot show the current ob:"+currentOb.getID());
                    }         
                }
				res = currentOb;
                break;
            }
		}
        if (res == null) {
            MsgManager.errDialog("Error: cannot find ob :" + paramObid+" in the obList");
        }
    	
    	return(res);
    }
    
    /*
     * Converts an array of OBScheduleInfo objects in a ObDataContainer array
     */
    public static ObDataContainer [] getOBContainers(Long [] idLongs , OBScheduleInfo [] obs, VisiPlotOptions visiplotOptions) {
    	ObDataContainer [] res = new ObDataContainer[idLongs.length];
    	for (int i = 0; i < res.length; i++) {
    		Long id = idLongs[i];    		
			res[i] = find(id.longValue(),obs,visiplotOptions).getOb();			
		}
    	
    	return(res);
    }
    
    /*
     * Converts an array of ObservationBlock objects in a ObDataContainer array
     */
    public static ObDataContainer [] getOBContainers(ObservationBlock [] obs) {
    	ObDataContainer [] res = new ObDataContainer[obs.length];
    	for (int i = 0; i < res.length; i++) {
			res[i] = new ObDataContainer(obs[i]);
			
		}
    	
    	return(res);
    }

    
    
    
    /**
     * Returns the ranking description string
     * 
     * @param obId
     * @param obs
     * @return
     */
    public static String getRankDescription(long obId , OBScheduleInfo [] obs) {
    	String result = "";
    	
    	for (int i = 0; i < obs.length; i++) {
			OBScheduleInfo info = obs[i];
			long id = info.getID();
			
			if (id == obId)
				return(info.getRankingDesc());
			
		}
    	return(result);
    }


    /**
     * @param now
     * @return String
     */
    public static String toVisiplotDate(Date now) {
        return formatter.format(now);
    }


	


	/**
	 * Create a new PCF which is identical to the pcf passed, except that
	 * all values have been multiplied by -1.0.
	 * See the header of this class for the reason why this is required.
	 */
	
	public static PCF setToNegative(PCF pcf) {
	    PCF rc = new PCF();
	    for (int i = 0; i < pcf.getIntervalCount(); i++) {
	        PCFInterval origValue = pcf.getIntervalAt(i);
	        PCFInterval negativeValue = new PCFInterval(origValue.getStart(),
	                origValue.getEnd(), origValue.getValue()*(-1.0));
	        rc.addInterval(negativeValue);
	    }
	    return rc;
	}
    
    
    
    
    
    
    

}
